#!/usr/bin/python
LICENSE_    = 'This software is covered by The LLU Licence 0.1'
VERSION_    = '0.6'
COPYRIGHT_    = 'Copyright Lars Lundqvist 2001, 2002'
PACKAGE_    = 'mamamua'
CONTACT_    = 'For information about this application contact:\n' + \
          'mamamua@lundqvist-zone.com'

from Tkinter import *

import tkFileDialog
import Pmw
import os
import getpass, imaplib, string, mailbox, re
import quopri
import base64
import StringIO
import mimify
import mimetools
import multifile
import smtplib
import ConfigParser
import binascii
import MimeWriter
import types
import tempfile
import time
import pickle

import socket  # socket.error 

import mailcap
import mimetypes


commands_text = """
--- General ---
Most actions can be performed by a right click on the actual object,
e.g. a message in the message list, the To/From/Cc field in the compose window, etc.

View mailbox content by left click on the desired mailbox.
View message by left click on the message you want to wiew.


--- Menu commands ---

Attachment:
\tSave\t\tClick on the attachment button
\tOpen\t\tRight click on the attachment button -> Open
\tOpen with\tRight click on attachment -> Open with -> Type command -> Open

New message:
Open menu Message -> New Message
        
Message:
Right click on message and select a menu item:
\tMove\t\tSelect destination folder by double click
\tDelete\t\tConfirm delete with Ok
\tReply\t\tEdit and send the reply message, Update To with right click in From-field
\t\t\tPick the desired original recipient from pop-up menu.
\tReply to all\tEdit and send response.
\tForward\t\tEdit and send
\tPrint
\tExport\t\tSelect file name of saved message
\tEdit\t\tOpen and edit any message, the message may saved again.
\tOpen in window\tView a message in a separate window.
\tDump raw\tView complete recieved message including message header.

Folder:
Right click on the folder and select menu item:
\tCreate\t\tType name of new mailbox, including path -> Ok
\tRename\t\tType new name, including path -> Ok
\tDelete\t\tOk to delete
\tSubscribe\tSelect mailbox -> Ok to subcribe (or double click)
\tCancel subsc\t ->Select from list -> Ok (or double clik)

Preferences:
\tEdit
\tSave
"""

#########################################################################
#
#########################################################################

class CollumnList(Pmw.MegaWidget):
    def __init__(self, parent, **kw):
        """
        List window with multiple resizabel collumns
        """

        optiondefs = (
                        ('collumns', [' '], Pmw.INITOPT),
                        ('headerclick', None, Pmw.INITOPT),
                        ('itemclick', None, Pmw.INITOPT),
                        ('itemclick3', None, Pmw.INITOPT),
                        ('itemdblclick', None, Pmw.INITOPT),
                        ('hull_width', 400, Pmw.INITOPT),
                        ('hull_height', 400, Pmw.INITOPT),
                        ('collumnmins', None, Pmw.INITOPT),
                        ('collumnmaxs', None, Pmw.INITOPT),
                        ('labeltext', 'Messages', Pmw.INITOPT)
                        )

        self.defineoptions(kw, optiondefs)

        Pmw.MegaWidget.__init__(self, parent)

        self.headline = Label(parent,  #self.pane.pane(name),
            text=self['labeltext'],
            anchor='n',
            justify=CENTER,
            relief=GROOVE,
            borderwidth=3,
            background='lightblue')

        self.headline.pack(side=TOP, fill=BOTH) 

        self.colnum = len(self['collumns']) 
        self.buttons = []
        self.listboxes = []
        self.cursel = [] # Allow multiple selection
        self.x = None
        self.y = None
        self.collumnsel = 0

        # print 'col hull_width hull_height', self['hull_width'], self['hull_height']

        self.pane = Pmw.PanedWidget(parent,
                        hull_width = self['hull_width'],
                        hull_height = self['hull_height'],
                        hull_borderwidth = 0,
                        separatorthickness = 0,
                        orient='horizontal')

        for i in range(self.colnum):
            s = 'c' + str(i)
            if self['collumnmins'] != None:
                mi = self['collumnmins'][i]
            else:
                mi = 100

            if self['collumnmaxs'] != None:
                ma = self['collumnmaxs'][i]
            else:
                ma = 1000

            self.pane.add(s, min = mi, max = ma)

        # Fixed width pane with  the scrollbar
        self.pane.add('sb', min= 20, max= 20)
        name = self.pane.panes()[self.colnum]
        self.button3=Label(self.pane.pane(name), text='')
        self.scrollbar = Scrollbar(self.pane.pane(name),orient=VERTICAL)

        # Set up the collumns
        for i in range(self.colnum):
            name = self.pane.panes()[i]

            self.buttons.append(
                    Label(self.pane.pane(name),
                        text=self['collumns'][i],
                        relief=SUNKEN,
                        anchor='w',
                        justify=LEFT) )

            self.listboxes.append(
                    Listbox(self.pane.pane(name),
                            yscrollcommand=self.scrollbar.set) )

            self.listboxes[i].config(width=60, bg='grey90', selectmode='extended')
            self.listboxes[i].bind('<ButtonRelease-1>', self._selection)
            self.listboxes[i].bind('<ButtonRelease-3>', self._selection3)
            self.listboxes[i].bind('<Double-Button-1>', self._dblselection)
            self.listboxes[i].bind('<Return>', self._dblselection)

            self.buttons[i].bind('<ButtonRelease-1>', self._execute)
            self.buttons[i].bind('<Return>', self._execute)
            self.buttons[i].pack(side=TOP,fill=X)
            self.listboxes[i].pack(side=LEFT, fill=Y)

        self.scrollbar.config(command=self._listScroll)
        self.scrollbar.pack(side=RIGHT, fill=Y)
        self.button3.pack(side=TOP,fill=X)
        self.pane.pack(expand = 1, fill='both')


    ### Public methods ###

    def insert(self, idx, rowdata):
        for i in range(self.colnum):
            if i >= len(rowdata):
                s = ''
            else:
                s = rowdata[i]
            self.listboxes[i].insert(idx, s)
        self.cursel = []


    def delete(self, first, last=None):
        if last == None:
            last = first
        for i in range(self.colnum):
            self.listboxes[i].delete(first, last)
        self.cursel = []


    def get(self, start_idx, end_idx = None):
        tot_res = []

        if end_idx == None:
            res = []
            for i in range(self.colnum):
                res.append(self.listboxes[i].get(start_idx))
            return res

        else:
            for j in range(self.listboxes[0].index(start_idx),
                           self.listboxes[0].index(end_idx)):
                res = []
                #print 'j', j
                if start_idx != None:
                    for i in range(self.colnum):
                        #print 'i',i
                        res.append(self.listboxes[i].get(j))
                        #print 'res', res
                tot_res.append(res)
                #print 'tot_res', tot_res

            return tot_res


    def find(self, col, key):
        found = 0
        for i in range(0, self.listboxes[0].index(END)):
            if self.listboxes[col].get(i) == key:
                found = 1
                break

        if found == 1:
            return i
        else:
            return None


    def curselection(self):
        return self.cursel

    def getcurselection(self):
        res = []
        try:
            for row in self.cursel:
                r1 = []
                for i in range(self.colnum):
                    r1.append( self.listboxes[i].get(row))
                res.append(r1)

        except (TclError, TypeError):
            res = []

        return res


    def configure(self, **kw):
        self.headline.configure(text = kw['labeltext'])


    def select_set(self, idx, end_idx=None):
        try:
            for i in range(self.colnum):
                self.listboxes[i].select_set(idx)
            self.listboxes[0].select_set(idx)
        except TclError:
            pass

        
        ### Private methods ###

    def _listScroll(self, cmd, step, unit=None):
        for i in range(self.colnum):
            self.listboxes[i].yview(cmd, step, unit)

    def _selection(self, event):
        self.x = event.x
        self.y = event.y

        sel = event.widget.curselection()

        for i in range(self.colnum):
            if self.listboxes[i] == event.widget:
                c = i
                break

        if sel == ():
            sel = None

        self.cursel = sel

        if (self['itemclick'] != None):
            self['itemclick'](self['collumns'][i], sel)

    def _selection3(self, event):
        self.x = event.x
        self.y = event.y

        if len(self.cursel) < 2:
            event.widget.select_clear(0,END)
            event.widget.select_set(event.widget.nearest(event.y))
        sel = event.widget.curselection()

        for i in range(self.colnum):
            if self.listboxes[i] == event.widget:
                break

        if sel == ():
            sel = None

        self.cursel = sel

        #print 'right click ', self['collumns'][i],  sel
        if self['itemclick3'] != None:
            self['itemclick3']()

    def _dblselection(self, event):
        self.x = event.x
        self.y = event.y

        sel = event.widget.curselection()

        for i in range(self.colnum):
            if self.listboxes[i] == event.widget:
                break

        if sel == ():
            sel = None

        self.cursel = sel

        #print 'Doubleclick ', self['collumns'][i],  sel
        if self['itemdblclick']() != None:
            self['itemdblclick']()

    def _execute(self, event):
        for i in range(self.colnum):
            if self.buttons[i] == event.widget:
                break
        self.collumnsel = i;

        if (self['headerclick'] != None):
            self['headerclick']()
        else:
            #print 'Button pressed, col ', i, ', ', event.widget.cget('text')
            pass


    def getxy(self):
        return self.x, self.y


    def yview(self, index):
        for j in range(self.colnum):
            self.listboxes[j].yview(index)

    def getCurrentCol(self):
        return self.collumnsel

    def sort(self, key_col):
        rows = {}
        key_list = []

        for i in range(0, self.listboxes[0].index(END)):
            row = []

            for j in range(self.colnum):
                row = row + [self.listboxes[j].get(i)]

            key = self.listboxes[key_col].get(i)

            # Make sure unique key
            while rows.has_key(key):
                key = key + '_'

            rows[key] = row
            key_list.append(key)

        key_list = rows.keys()
        key_list.sort()
        self.delete(0, END)

        for k in key_list:
            self.insert(END, rows[k])


###########################################################################
#
###########################################################################

class AddressAppender:
    def __init__(self, widget_, address_):
        self.widget_ = widget_
        self.address_ = address_

    def execute(self):
        self.widget_.insert(END, ',' + self.address_)
        del(self)


###########################################################################
#
###########################################################################

class AddressSwapper:
    def __init__(self, widget_, address_):
        self.widget_ = widget_
        self.address_ = address_

    def execute(self):
        self.widget_.delete(0, END)
        self.widget_.insert(END, self.address_)
        del(self)


###########################################################################
#
###########################################################################

class ComposeMail(Pmw.MegaWidget):
    """
    Compose and send mail via SMTP.
    """
    def __init__(self, parent, **kw):

        optiondefs = (
           ('toAddress', '', Pmw.INITOPT),
           ('toSubject', '', Pmw.INITOPT),
           ('extraAddress', None, Pmw.INITOPT),
           ('attachments', '', Pmw.INITOPT),
           ('embeddedobjects', [], Pmw.INITOPT),
           ('messagetext', None, Pmw.INITOPT),
           ('mailserver', None, Pmw.INITOPT),
           ('confdir', None, Pmw.INITOPT),
           ('signatureflag', 1, Pmw.INITOPT),
            )

        self.defineoptions(kw, optiondefs)
        Pmw.MegaWidget.__init__(self, parent)

        self.readConf()
        
        self.parent = parent
        self.fn = ''
        self.M = self['mailserver']
        self.importLastAddress()
        self.addLastAddressTab = [self.addLastAddress0,
                             self.addLastAddress1,
                             self.addLastAddress2,
                             self.addLastAddress3]

        self.extraballoon = None
        self.focusWidget = None


        # Create the compose window
        self.composeWindow = Toplevel()

        self.headerFrame = Frame(self.composeWindow)

        # Get alternate from addresses from address book
        abook = AddressBook(self.parent, confdir=self['confdir'], mode='command')
        l =  abook.queryKey('altfrom', 1)
        if l != None:
            self.alternatefrom = []
            for s in l:
                m = s + ' <' + abook.readKey(s, 'email') + '>'
                self.alternatefrom = self.alternatefrom + [m]
        else:
            self.alternatefrom = None
        
        # Get full name if inay from address book
        s = self.user + '@' + self.domain
        u = abook.query(s)
        abook.destroy()
        if u != None:
            s = u

        self.fromWidget = Pmw.EntryField(self.headerFrame,
                labelpos = 'w',
                label_text = _('From:'),
                value = s,
                validate = None,
                command = self.execute)

        self.fromWidget._entryFieldEntry.bind(
                           '<ButtonRelease-3>', self.fromExecuteButton3)

        self.fromWidget.pack(expand = 1, fill = 'both', padx = 0, pady = 0)

        self.toWidget = Pmw.EntryField(self.headerFrame,
                labelpos = 'w',
                label_text = _('To:'),
                value = self['toAddress'],
                validate = None,
                command = self.execute)

        self.toWidget._entryFieldEntry.bind(
                    '<Double-Button-1>', self.execute)
  
        self.toWidget._entryFieldEntry.bind(
                    '<ButtonRelease-3>', self.toExecuteButton3)

        self.toWidget.pack(expand = 1, fill = 'both', padx = 0, pady = 0)

        self.ccWidget = Pmw.EntryField(self.headerFrame,
                labelpos = 'w',
                label_text = _('Cc:'),
                validate = None,
                command = self.execute)

        self.ccWidget._entryFieldEntry.bind(
                    '<Double-Button-1>', self.ccexecute)

        self.ccWidget._entryFieldEntry.bind(
                    '<ButtonRelease-3>', self.ccExecuteButton3)

        self.ccWidget.pack(expand = 1, fill = 'both', padx = 0, pady = 0)

        self.subjectWidget = Pmw.EntryField(self.headerFrame,
                labelpos = 'w',
                label_text = _('Subject:'),
                value = self['toSubject'],
                validate = None,
                command = self.execute)

        self.subjectWidget.pack(expand = 1, fill = 'both', padx = 0, pady = 0)

        self.headerFrame.pack(expand = 0, fill = 'x', padx = 0, pady = 0)

        self.textWidget= Pmw.ScrolledText(self.composeWindow,
                borderframe = 1,
                labelpos = 'n',
                label_text=_('Messsage content'),
                usehullsize = 1,
                hull_width = 600,
                hull_height = 300,
                text_padx = 0,
                text_pady = 0,
                text_wrap='word',
                text_width = 80
            )


        self.attachmentWidget = Pmw.EntryField(self.composeWindow,
                labelpos = 'w',
                label_text = _('Attachments:'),
                value = self['attachments'],
                validate = None,
                command = self.attachmentCmd)

        if (self['messagetext'] != None) :
            self.textWidget.insert(END, self['messagetext'])

        self.menubar = Menu(self.composeWindow)
        self.menubar.configure(background='lightblue')

        self.filemenu = Menu(self.menubar, tearoff = 0)
        self.filemenu.add_command(label = _('Print'), command = self.mailprint)
        self.filemenu.add_separator()
        self.filemenu.add_command(label = _('Exit'), command = self.composeWindow.destroy)
        self.menubar.add_cascade(label = _('File'), menu = self.filemenu)

        self.addressMenu = Menu(self.menubar, tearoff = 1,
                                postcommand = self.updateAddressMenu)

        if self['extraAddress'] != None:
            self.extraMenu = Menu(self.addressMenu)
            self.extraMenu.add_command(
                              label = _('All original recipients'),
                              command = self.toInsertExtra)

            self.extraballoon = Pmw.Balloon(self.addressMenu)
            l = string.split(self['extraAddress'], ',')
            #print self['extraAddress'], l
            s = ''
            for s1 in l:
                s1 = string.strip(s1)
                s = s + s1 + '\n'
                a =  AddressAppender(self.toWidget, s1)
                self.extraMenu.add_command(label =  s1,
                                         command = a.execute)


            self.extraballoon.bind(self.extraMenu, s)

        self.menubar.add_cascade(label = _('Address'), menu = self.addressMenu)

        # Add GPG menu if GPG available
        if gpgVersion() != None:
            #print 'gpg', gpgVersion()
            self.pgpmenu = Menu(self.menubar, tearoff = 0)
            self.pgpmenu.add_command(label = _('Sign'),
                                        command = self.pgpsign)
            self.pgpmenu.add_command(label = _('Encrypt'),
                                        command = self.pgpencrypt)
            self.pgpmenu.add_command(label = _('Sign and encrypt'),
                                        command = self.pgpsignencrypt)
            self.menubar.add_cascade(label = 'GnuPG', menu = self.pgpmenu)

        self.mailmenu = Menu(self.menubar, tearoff = 0)
        self.mailmenu.add_command(label = _('Send'), command = self.send)
        self.mailmenu.add_command(label = _('Save in folder ')+ self.draftfolder,
                                  command = self.saveinfolder)
        self.menubar.add_cascade(label = _('Transfer'), menu = self.mailmenu)

        self.attachmenu = Menu(self.menubar, tearoff = 0)
        self.attachmenu.add_command(label = _('Add attachment'),
                                    command = self.attachmentDialog)
        self.menubar.add_cascade(label = _('Attachment'), menu = self.attachmenu)

        self.composeWindow.config(menu = self.menubar)
        
        self.fromMenu = Menu(self.menubar, tearoff = 1)
        a =  AddressSwapper(self.fromWidget, self.fromWidget.get())
        self.fromMenu.add_command(label = self.fromWidget.get(),
                                  command = a.execute)

        # Add alterante from addresses to from menu
        if self.alternatefrom != None:
            abook = AddressBook(self.parent, confdir=self['confdir'], mode='command')

            if self['extraAddress'] != None:
                l = string.split(self['extraAddress'], ',')
                prefuser = self.user + '@' + self.domain
                s1 = abook.query(prefuser)
                if s1 != None:
                    #print 'pref user', s1
                    prefuser = s1
                prefuserfl = 0
            else:
                l = None
            i = 1
            for s in self.alternatefrom:
                s1 = abook.query(s)
                if s1 != None:
                    s = s1
              
                if len(s) > 0:
                    a =  AddressSwapper(self.fromWidget, s)
                    self.fromMenu.add_command(label = chr(0x30 + i) + '. ' + s,
                                         command = a.execute)
                    i = i + 1

                    try:
                        for s1 in l:
                            s1 = string.strip(s1)
                            #print  '\'' + s + '\'',  '\'' + s1 + '\''
                            if (s == s1) and (prefuserfl == 0):
                                self.fromWidget.delete(0, END)
                                self.fromWidget.insert(END, s1)
                                if s1 == prefuser:
                                    prefuserfl = 1

                    except TypeError:
                        pass

            abook.destroy()

        # Signature to append to message (not the GPG signature)
        self.signFrame = Frame(self.composeWindow)
        self.signflag = IntVar()
        self.signcheck = Checkbutton(self.signFrame,
                                        text=_('Append signature to message body'),
                                        justify=LEFT,
                                        anchor='w',
                                        command=self.signcmd,
                                        variable=self.signflag,)

        self.textWidget.pack(expand = 1, fill = 'both')
        self.signFrame.pack(side='top', fill='both', padx = 0, pady = 0)
        self.signcheck.pack(side='top', padx = 0, pady = 0)
        self.attachmentWidget.pack(expand = 0, fill = 'x', padx = 0, pady = 0)


    def readConf(self):
        try:
            self.user =  conf.get("SMTP", "user")
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            self.user = getpass.getuser()
            print "SMTP user same as login: ", self.user
            if hasattr(conf, 'set'):
                conf.set('SMTP', 'user', self.user)

        try:
            self.host =  conf.get("SMTP", "host")
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            self.host = 'localhost'
            print 'Default SMTP host ', self.host
            if hasattr(conf, 'set'):
                conf.set('SMTP', 'host', self.host)
        try:
            self.domain =  conf.get("SMTP", "domain")
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            print "Default SMTP domain: localdomain"
            self.domain = "localdomain"
            if hasattr(conf, 'set'):
                conf.set('SMTP', 'domain', self.domain)
        try:
            self.replyTo =  conf.get("SMTP", "replyto")
            if self.replyTo == '-':
                self.replyTo = None
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            print 'No default Reply-To'
            self.replyTo = None
            if hasattr(conf, 'set'):
                conf.set('SMTP', 'replyto', '-')

        try:
            self.thread =  conf.get("SMTP", "thread")
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            print 'Default use SMTH thread'
            self.thread = 'yes'
            if hasattr(conf, 'set'):
                conf.set('SMTP', 'thread', self.thread)            

        try:
            self.draftfolder =  conf.get("MAIN", "draftfolder")
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            print "Default draftfolder: ~\/mail\/Utkast'"
            self.draftfolder = '~/mail/Utkast'
            if hasattr(conf, 'set'):
                conf.set('MAIN', 'draftfolder', self.draftfolder)            

        try:
            self.sentfolder =  conf.get("MAIN", "sentmailfolder")
            if self.sentfolder == '-':
                self.sentfolder = None
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            s = '~/mail/Sent-mail'
            if self['mailserver'].checkFolderSubscription(s) != None:
                self.sentfolder = s
                if hasattr(conf, 'set'):
                    conf.set('MAIN', 'sentmailfolder', self.sentfolder)
            else:
                print "No default  sent mail folder"
                self.sentfolder = None
                if hasattr(conf, 'set'):
                    conf.set('MAIN', 'sentmailfolder', '-')

        
    def signcmd(self):
        if self.signflag.get() == 1:
            self.signText= Pmw.ScrolledText(self.signFrame,
                borderframe = 1,
                usehullsize = 1,
                labelpos = 's',
                label_text = None,
                hull_width = 600,
                hull_height = 100,
                text_padx = 0,
                text_pady = 0,
                text_wrap='word',
                text_width = 80,
                )
            self.signText.insert(END, self.readSignature())
            self.signText._textbox.config(state = DISABLED)
            self.signText.pack(side='bottom', padx = 0, pady = 0)
        else:
            self.signText.destroy()


    def mailprint(self):
        print 'Compose print not implemented yet'
        pass


    def execute(self, event = None):
        if event != None:
           self.focusWidget = event.widget
        #print "Execute"

        d = ''
        if self.focusWidget == self.ccWidget:
            d = 'Cc'
        if self.focusWidget == self.toWidget:
            d = 'To'

        a = AddressBook(self.composeWindow,
                        exitcallback = self.insertTo,
                        confdir=self['confdir'],
                        dest = d)


    def fromExecuteButton3(self, event):
        #print 'Button3'
        self.fromMenu.post(self.fromWidget.winfo_rootx()+ 10,
                              self.fromWidget.winfo_rooty() + 20)


    def fromMenuExecute(self):
        #print 'from'
        self.fromWidget.delete(0, END)
        self.fromWidget.insert(END, self['extraAddress'])


    def toExecuteButton3(self, event):
        #print 'Button3'
        self.focusWidget = self.toWidget
        self.addressMenu.post(self.toWidget.winfo_rootx()+ 10,
                              self.toWidget.winfo_rooty() + 20)


    def ccexecute(self, event = None):
        if event != None:
            self.focusWidget = event.widget
        if self['extraAddress'] != None:
            self.ccWidget.insert(END,',')
            self.ccWidget.insert(END, self['extraAddress'])


    def ccExecuteButton3(self, event):
        #print 'Button3'
        self.focusWidget = self.ccWidget
        self.addressMenu.post(self.ccWidget.winfo_rootx()+ 10,
                              self.ccWidget.winfo_rooty() + 20)


    def toInsertExtra(self, event = None):
        if event != None:
            if self['extraAddress'] != None:
               event.widget.insert(END,',')
               event.widget.insert(END, self['extraAddress'])       
        else:     
            if self['extraAddress'] != None:
                if self.focusWidget != None:
                   self.focusWidget.insert(END,',')
                   self.focusWidget.insert(END, self['extraAddress'])
                else:
                   self.toWidget.insert(END,',')
                   self.toWidget.insert(END, self['extraAddress'])


    def insertTo(self, s):
        if self.focusWidget != None:
            if len(self.focusWidget.get()) != 0:
                self.focusWidget.insert(END,',')
            self.focusWidget.insert(END, s)
        else:
            if len(self.toWidget.get()) != 0:
                self.toWidget.insert(END,',')
            self.toWidget.insert(END, s)


    def addLastAddress0(self, event = None):
        self.insertTo(self.lastaddress[0])
        if event != None:
            print self.lastaddress[0]


    def addLastAddress1(self, event = None):
        self.insertTo(self.lastaddress[1])
        if event != None:
            print self.lastaddress[1]


    def addLastAddress2(self, event = None):
        self.insertTo(self.lastaddress[2])
        if event != None:
            print self.lastaddress[2]


    def addLastAddress3(self, event = None):
        self.insertTo(self.lastaddress[3])
        if event != None:
            print self.lastaddress[3]


    def pushLastAddress(self, s):
        newflag = 1
        for str in self.lastaddress:
            if str == s:
                newflag = 0
                break

        if (newflag == 1):
            tmp = []
            tmp.append(s)
            tmp.append(self.lastaddress[0])
            tmp.append(self.lastaddress[1])
            tmp.append(self.lastaddress[2])
            del self.lastaddress
            self.lastaddress = tmp
            #print 'push ', s, ', ', self.lastaddress

        b = ''
        for a in self.lastaddress:
            b = b + ',' + a


    def importLastAddress(self):
        fn = self['confdir'] +'mamamua/lastadresses'
        try:
            fp = open(fn)
            self.lastaddress = pickle.load(fp)
            fp.close()
        except IOError:
            self.lastaddress = ['','','','']


    def readSignature(self):
        k = self.getAddrAlias(self.fromWidget.get())
        # print 'read signature file', self.fromWidget.get() , k

        if k != None:
            abook = AddressBook(self.parent, confdir=self['confdir'], mode='command')
            signaturefn = abook.readKey(k, 'sign')
            abook.destroy()
            if signaturefn == None:
                return ''
            try:
                fp = open(signaturefn)
                s = fp.read()
                fp.close()
            except IOError:
                s = ''        
        else:
            s = ''

        return s
        

    def getAddrAlias(self, addr):
        mo = re.match('(.*) <+.*', addr)
        if mo != None:
            ret = re.sub('\"', '',  mo.groups()[0])
        else:
            ret = None
        return ret


    def exportLastAddress(self):
        fn = self['confdir'] + 'mamamua/lastadresses'
        try:
            fp = open(fn,'w')
        except IOError:
            fp = os.open(fn, os.O_CREAT)
            os.close(fp)
            fp = open(fn,'w')

        if fp != None:
            pickle.dump(self.lastaddress, fp)
            fp.close()
            status = 'Ok'
        else:
            status = 'Error'

        return status
  
 
    def updateAddressMenu(self):
        self.addressMenu.delete(0, END)

        self.addressMenu.add_command(label = _('Addressbook'),
                                    command = self.execute)

        #print 'extra ', self['extraAddress']
        if self['extraAddress'] != None:
            self.addressMenu.add_separator()
            self.addressMenu.add_cascade(label=_('Original recipients'), menu=self.extraMenu)

        if self.lastaddress != None:
            #print 'new menu ',self.lastaddress
            self.addressMenu.add_separator()
            i = 0
            for s in self.lastaddress:
                #print s
                if len(s) > 40:
                    s = s[0:40] + '...'
                self.addressMenu.add_command(label = chr(0x30 + i) + '. ' + s,
                                             command = self.addLastAddressTab[i])
                i = i + 1


    def saveinfolder(self):
        s = 'To: ' + self.toWidget.get()
        s = s + '\r\nCc: ' + self.ccWidget.get()
        if  len(self.subjectWidget.get()) != 0:
            s = s + '\r\nSubject: ' + self.subjectWidget.get()
        else:
            s = s + '\r\nSubject: ' + _('DRAFT')
        s = s + '\r\n' + self.textWidget.get()

        self.M.saveMessage(self.draftfolder, [s])
        self.composeWindow.destroy()


    def pgpencrypt(self):
        toaddr = string.split(self.toWidget.get(),',')
        if (len(toaddr) == 1) and (toaddr[0] != ''):
            rec = self.toWidget.get()
        else:
            rec = None

        ks = StringVar()
        pwidg = SelectPGPKeyWidget(self.parent, ks, rec)
        # print 'Selected key', ks.get()
        toaddr = ks.get()

        s = self.encryptText(self.textWidget.get(), toaddr)
        if s != None:
            self.textWidget.delete(0.0,END)
            self.textWidget.insert(END, s)


    def pgpsignencrypt(self):
        toaddr = string.split(self.toWidget.get(),',')
        if (len(toaddr) == 1) and (toaddr[0] != ''):
            rec = self.toWidget.get()
        else:
            rec = None

        ks = StringVar()
        pwidg = SelectPGPKeyWidget(self.parent, ks)
        # print 'Selected key', ks.get()
        toaddr = ks.get()
        
        s = self.signEncryptText(self.textWidget.get(), toaddr)
        if s != None:
            self.textWidget.delete(0.0,END)
            self.textWidget.insert(END, s)


    def pgpsign(self):
        s = self.signText(self.textWidget.get())
        if s != None:
            self.textWidget.delete(0.0, END)
            self.textWidget.insert(END, s)


    def send(self):
        s = string.split(self.toWidget.get(),',')
        if s == ['']:
            return
        else:
            c = string.split(self.ccWidget.get(),',')
            if c != ['']:
                s = s + c
                ccaddr = ''
                for a in c:
                    a1 = t_mimeaddr(a)
                    if ccaddr != '':
                        ccaddr = ccaddr + ', '
                    ccaddr = ccaddr + a

            toaddr = []
            for a in s:
                a1 = t_mimeaddr(a)
                toaddr = toaddr + [a1]
 
        self.pushLastAddress(self.toWidget.get())
        self.exportLastAddress()

        s = self.fromWidget.get()
        fromaddr = t_mimeaddr(s)

        s = self.subjectWidget.get()
        subject = t_mime(s)

        header = ''
        header = 'From: ' + fromaddr + '\r\n'
        header = header + 'To: ' + self.toWidget.get() + '\r\n'
        if self.ccWidget.get() != '':
            header = header + 'Cc: ' + ccaddr + '\r\n'
        header = header + 'Subject: ' + subject + '\r\n'
        header = header + 'X-Mailer: Mama Mua ' + VERSION_ + '\r\n'

        if self.replyTo != None:
            header = header + 'Reply-To: ' + self.replyTo + '\r\n'

        msg = header

        if (self.attachmentWidget.get() != '') or (self['embeddedobjects'] != []):
            #print 'Add attachments ', self.attachmentWidget.get()
            self.fp = StringIO.StringIO()
            self.mo = MimeWriter.MimeWriter(self.fp)
            self.body = self.mo.startmultipartbody('mixed')

            self.mp = self.mo.nextpart()
            self.sub_body = self.mp.startbody( 'text/plain',
                                        [('charset',
                                         'iso-8859-1')] )
            self.sub_body.write(self.textWidget.get())

            for att in string.split(self.attachmentWidget.get(),';'):
                if att == '':
                    continue

                #print 'Add attachment: ', att

                self.mp = self.mo.nextpart()
                self.mp.addheader('Content-Transfer-Encoding','base64')

                self.mp.addheader('Content-Disposition', \
                     'attachment;\r\n        filename=\"' + att + '\"') 
                self.sub_body = self.mp.startbody('octet-stream',
                    [('name', att)] )
 
                self.a_fp = open(att)
                self.tmp = StringIO.StringIO()
                base64.encode(self.a_fp, self.tmp)
                self.tmp.seek(0)
                self.sub_body.write(self.tmp.read())

            for eo in self['embeddedobjects']:
                text, codec, data = eo.getEncoded()
                # print 'Add attachment', text, codec
                self.mp = self.mo.nextpart()
                s = 'plain/text'
                if codec == base64:
                    s = 'base64'
                if codec == quopri:
                    s = 'quoted-printable'
                self.mp.addheader('Content-Transfer-Encoding', s)

                self.mp.addheader('Content-Disposition', \
                     'attachment;\r\n        filename=\"' + text + '\"')
                self.sub_body = self.mp.startbody('octet-stream',
                    [('name', text)] )

                self.sub_body.write(data)

            self.mo.lastpart()

            self.fp.seek(0)
            msg = msg + self.fp.read()
        else:
            msg = msg + '\r\n' + self.textWidget.get()

        if self.signflag.get() == 1:
            msg = msg + self.signText.get()

        import thread
        if self.thread == 'no':
            self.sendThread(self.parent,
                            self.host,
                            fromaddr, 
                            toaddr, 
                            msg)
        else:
            thread.start_new_thread(self.sendThread,
                           (self.parent,
                            self.host,
                            fromaddr,
                            toaddr,
                            msg))

        if self.sentfolder != None:
            #print 'save', self.sentfolder
            self.M.saveMessage(self.sentfolder, ['From: ' + fromaddr + '\r\n' + msg])

        self.composeWindow.destroy()


    def sendThread(self, parent, host, fr, to, msg):
        import smtplib
        # print 'host ', host, ', fr ', fr, ', to ', to, ', msg ',msg
        text = _('SMTP Error')
        try:
            smtpServer = smtplib.SMTP(host)
            smtpServer.sendmail(fr, to, msg)
            smtpServer.quit()
            text = _('Message sent to ') + to[0] + '...'

        except smtplib.SMTPConnectError:
            text = _('SMTP Connect error')
        except smtplib.SMTPHeloError:
            text = _('SMTP HELO error')
        except smtplib.SMTPSenderRefused:
            text = _('SMTP sender refused')
        except smtplib.SMTPRecipientsRefused:
            text = _('SMTP recipients refused')
        except smtplib.SMTPDataError:
            text = _('SMTP data (message contents) error')

        d = Pmw.MessageDialog(parent,
                              title = 'SMTP status',
                              defaultbutton = 0,
                              message_text =  text
                              )
        d.iconname('SMTP status')
        d.activate()


    def attachmentCmd(self):
        #print 'Attachment ', self.attachmentWidget.get()
        pass


    def attachmentDialog(self):
        fn=tkFileDialog.askopenfilename(filetypes=[("all filez", "*")],
                                        title=_('Add attachment'))
        if (fn != None) and (len(fn) != 0):
           self.attachmentWidget.setentry(self.attachmentWidget.get() + ';' + fn)


    def textdbclick(self):
        #print 'Double click'
        pass


    def errorAlert(self, s):
        self.errorDialog = Pmw.MessageDialog(self.composeWindow,
        title = _('Error'),
        defaultbutton = 0,
        message_text = s 
            )
        self.errorDialog.iconname('Error')
        self.errorDialog.activate()


    def encryptText(self, text, recipient):
        self.gpgCmd = 'gpg --yes --no-secmem-warning -eaqr'
        fn = tempfile.mktemp()
        f = os.open(fn, os.O_RDWR | os.O_CREAT)
        os.write(f, text)
        os.close(f)
        s =  os.system(self.gpgCmd + ' ' + recipient + ' ' + fn)
        if s != 0:
            return None
        os.unlink(fn)
        f = os.open(fn + '.asc', os.O_RDWR)
        enctext = os.read(f,1000000)
        os.close(f)
        return enctext


    def signEncryptText(self, text, recipient):
        self.gpgCmd = 'gpg --yes --no-secmem-warning --passphrase-fd 0 -esaqt'
        
        self.pwdialog = Pmw.PromptDialog(self.parent,
            title = _('PGP'),
            label_text = _('Enter PGP passphrase:'),
            entryfield_labelpos = 'n',
            entry_show = '*',
            defaultbutton = 0,
            buttons = ('OK', 'Cancel'),
            command = self.pwExecute)
        
        self.pwdialog.activate()
        if self.pw == None:
            return None
        
        fn1 = tempfile.mktemp()
        f1 = os.open(fn1, os.O_RDWR | os.O_CREAT)
        os.write(f1, text)
        os.close(f1)
        fn2 = tempfile.mktemp()
        f2 = os.open(fn2, os.O_RDWR | os.O_CREAT)
        os.close(f2)
        if self.pw != None:
            c = os.popen('echo '+self.pw+ '|' +self.gpgCmd+' -o '+fn2+' -r '+recipient+' '+fn1)
            signtext = c.read()
            c.close()
        else:
            signtext = ''
        os.unlink(fn1)
        f2 = os.open(fn2, os.O_RDWR)
        signtext = os.read(f2, 1000000)
        os.close(f2)
        os.unlink(fn2)
        
        return signtext


    def signText(self, text):
        self.gpgCmd = 'gpg --yes --no-secmem-warning --passphrase-fd 0 -saqt'

        self.pwdialog = Pmw.PromptDialog(self.parent,
            title = _('PGP'),
            label_text = _('Enter PGP passphrase:'),
            entryfield_labelpos = 'n',
            entry_show = '*',
            defaultbutton = 0,
            buttons = ('OK', 'Cancel'),
            command = self.pwExecute)

        self.pwdialog.activate()
        if self.pw == None:
            return None

        fn1 = tempfile.mktemp()
        f1 = os.open(fn1, os.O_RDWR | os.O_CREAT)
        os.write(f1, text)
        os.close(f1)
        fn2 = tempfile.mktemp()
        f2 = os.open(fn2, os.O_RDWR | os.O_CREAT)
        os.close(f2)
        if self.pw != None:
            c = os.popen('echo '+self.pw+ '|' +self.gpgCmd+' -o '+fn2+' '+fn1)  
            signtext = c.read()
            c.close()
        else:
            signtext = ''
        os.unlink(fn1)
        f2 = os.open(fn2, os.O_RDWR)
        signtext = os.read(f2, 1000000)
        os.close(f2)
        os.unlink(fn2)

        return signtext


    def pwExecute(self, result):
        if result is None or result == 'Cancel':
            self.pwdialog.deactivate(result)
            self.pw = None
        else:
            self.pw = self.pwdialog.get()
            self.pwdialog.deactivate()


###########################################################################
#
###########################################################################

def sendThread(host, fr, to, msg):
    import smtplib

    root = Toplevel()
    
    # print 'host ', host, ', fr ', fr, ', to ', to, ', msg ',msg
    text = _('SMTP Error')
    try:
        smtpServer = smtplib.SMTP(host)
        smtpServer.sendmail(fr, to, msg)
        smtpServer.quit()
        text = _('Message sent to ') + to[0] + '...'

    except smtplib.SMTPConnectError:
        text = _('SMTP Connect error')
    except smtplib.SMTPHeloError:
        text = _('SMTP HELO error')
    except smtplib.SMTPSenderRefused:
        text = _('SMTP sender refused')
    except smtplib.SMTPRecipientsRefused:
        text = _('SMTP recipients refused')
    except smtplib.SMTPDataError:
        text = _('SMTP data (message contents) error')

    d = Pmw.MessageDialog(root,
                          title = 'SMTP status',
                          defaultbutton = 0,
                          message_text =  text
                          )
    d.iconname('SMTP status')
    d.activate()

    root.destroy()
    
    
###########################################################################
#
###########################################################################

class Embedded_object(Button):      
    """
    Attachment container
    """
    def __init__(self, parent, text_, data_, codec):
        Button.__init__(self, parent)
        self.parent = parent
        self.text_ = t_unmime(text_)
        self.data_ = data_
        self.codec = codec
        self.configure(text = self.text_, command = self.execute)
        self.bind('<ButtonRelease-3>', self.buttonCmd)


    def execute(self):
        afile = StringIO.StringIO(self.data_)
        self.saveAttachDialog(self.text_, afile, self.codec)


    def buttonCmd(self, w=None):

        self.f_ext = mimeGetFileExt(self.text_)
        self.temp_fn = self.createTempFile(self.f_ext)
        
        prog = re.compile('.*filename=\"+(.*)\"', re.DOTALL)
        mo = prog.match(self.text_)
        if mo != None:
            fn = mo.groups()[0]
            self.mimeStr = getMimeCmd(fn, self.temp_fn)
        else:
            self.mimeStr = None

        if self.mimeStr == '':
            self.mimeStr = None

        if os.name == 'nt':
            self.mimeStr = fn
        
        if self.mimeStr != None:
            b = ('Open', 'Open with ...', 'Cancel')
        else:
            b = ('Open with ...', 'Cancel')
            
        self.dialog = Pmw.Dialog(self.parent,
            title = _('Open embedded MIME object'),
            defaultbutton = 0,
            buttons = b,
            command = self.executeButtonCmd)

        if self.mimeStr != None:
            self.label_ = Label(self.dialog.interior(),
                                anchor = W,
                                justify = LEFT,
                                text = _('Default open command: ') + self.mimeStr
                                )
            self.label_.pack(expand = 1, fill = 'both', padx = 0, pady = 0)

        self.dialog.activate()


    def executeButtonCmd(self, result):
        if result is None or result == 'Cancel':
            self.dialog.deactivate(result)
            os.unlink(self.temp_fn)
        elif result == 'Open':
            self.dialog.deactivate()
            import thread
            thread.start_new_thread(mimeExec, (self.temp_fn,))

        else:
            self.customdialog = Pmw.Dialog(self.parent,
                title = _('Open embedded MIME object'),
                defaultbutton = 0,
                buttons = ('Open with...', 'Cancel'),
                command = self.executeCustomButtonCmd)
            
            self.entry_ = Pmw.EntryField(self.customdialog.interior(),
                labelpos = 'w',
                label_text = _('Custom open with'),
                value = '',
                )

            self.entry_.pack(expand = 1, fill = 'both', padx = 0, pady = 0)

            self.dialog.deactivate()
            self.customdialog.activate()


    def executeCustomButtonCmd(self, result):
        if result is None or result == 'Cancel':
            self.customdialog.deactivate(result)
        else:
            cmd = self.entry_.get()
            self.customdialog.deactivate()
            # get extension from self.text_ as par to mktemp
            prog = re.compile('.*filename=.*\.(.*)\"', re.DOTALL)
            mo = prog.match(self.text_)
            if mo != None:
                ext = '.' + mo.groups()[0]
            else:
                ext = ''
            fn = tempfile.mktemp(ext)
            afile = open(fn, "wb")
            text = re.sub( '=\r\n', '', self.data_)
            sfile = StringIO.StringIO(text)
            self.codec.decode(sfile, afile)
            afile.close()

            os.system(cmd + ' ' + fn)
            os.unlink(fn)


    def createTempFile(self, ext='tmp'):
            fn = tempfile.mktemp(ext)
            afile = open(fn, "wb")
            text = re.sub( '=\r\n', '', self.data_)
            sfile = StringIO.StringIO(text)
            self.codec.decode(sfile, afile)
            afile.close()
            return fn
            
        

    def saveAttachDialog(self, header, file, codec):
        ftext = t_getfilname(header)
        fn=tkFileDialog.asksaveasfilename(filetypes=[("all filez", "*")],
                                        initialfile=ftext)
        
        if (fn != None) and (len(fn) != 0):
            #print 'Save as', fn
            afile = open(fn, "wb")
            s = self.saveToFile(afile)

    def saveToFile(self, outfile):
        text = re.sub( '=\r\n', '', self.data_)
        tmpfile = StringIO.StringIO(text)
        self.codec.decode(tmpfile, outfile)
        outfile.close()
        return self.text_

    def getEncoded(self):
       return self.text_, self.codec, self.data_



###########################################################################
#
###########################################################################

class Embedded_PGP_KEY_object(Button):
    """
    PGP key container
    """
    def __init__(self, parent, data_):
        Button.__init__(self, parent)
        self.data_ = data_
        self.configure(text = _('PGP key'), command = self.execute,
                         activebackground='red')


    def execute(self):
        ftext = 'pgpkey.asc'
        fn=tkFileDialog.asksaveasfilename(filetypes=[("all filez", "*")],
                                        initialfile=ftext)

        if (fn != None) and (len(fn) != 0):
            #print 'Save as', fn
            f = os.open(fn, os.O_RDWR | os.O_CREAT)
            os.write(f, self.data_)
            os.close(f)


###########################################################################
#
###########################################################################

class Embedded_PGP_object(Button):
    """
    PGP encrypted data widget
    """

    def __init__(self, parent, data_, gpgCmd=None):
        Button.__init__(self, parent)
        self.data_ = data_
        self.parent = parent
        self.pw = None
        if gpgCmd == None:
            self.gpgCmd = 'gpg --decrypt --passphrase-fd 0 --batch'
        else:
            self.gpgCmd = gpgCmd
        self.configure(text = _('PGP encrypted object'), command = self.execute,
                                 activebackground='red')


    def execute(self):
        s =  self.decryptText(self.data_)

        po = re.compile('-----BEGIN PGP MESSAGE-----.*-----END PGP MESSAGE-----', re.DOTALL)
        mo = po.match(s)
        if mo != None:
            s =  self.decryptText(mo.group())

        if len(s) == 0:
            s = self.data_
            t = _('Encrypted text')
        else:
            t = _('PGP decrypted text')
        dialog = Pmw.TextDialog(self.parent, scrolledtext_labelpos = 'n',
                                title = 'Mama Mua',
                                defaultbutton = 0,
                                label_text = t )
        dialog.insert('end', s)


    def decryptText(self, enctext):
        if self.pw == None:
            self.pwdialog = Pmw.PromptDialog(self.parent,
                title = _('PGP'),
                label_text = _('Enter PGP passphrase:'),
                entryfield_labelpos = 'n',
                entry_show = '*',
                defaultbutton = 0,
                buttons = ('OK', 'Cancel'),
                command = self.pwExecute)

            self.pwdialog.activate()

        fn = tempfile.mktemp()
        f = os.open(fn, os.O_RDWR | os.O_CREAT)
        os.write(f, enctext)
        os.close(f)
        if self.pw != None:
            c = os.popen('echo ' + self.pw + '|' + self.gpgCmd + ' ' + fn)
            dectext = c.read()
            c.close()
        else:
            dectext = ''
        os.unlink(fn)
        return dectext


    def pwExecute(self, result):
        if result is None or result == 'Cancel':
            self.pwdialog.deactivate(result)
            self.pw = None
        else:
            self.pw = self.pwdialog.get()
            self.pwdialog.deactivate()



    
############################################################################
#
############################################################################

class NullMailServer(Pmw.MegaWidget):
    """
    Null mail server for offline debug of GUI etc.
    """

    def __init__(self, parent, **kw):
        Pmw.MegaWidget.__init__(self, parent)

    def login(self):
        pass

    def logout(self):
        pass

    def getStatus(self):
        return 'UP'

    # Return the directory hierarchy separator, used in path names
    def getHierachySeparator(self):
        return '/'

    def getFolderOption(self, m, opt):
        return None
    
    # Get a list containing the contents of mail folder 'mnum'
    def getFolderContent(self, mbox):
        flist = []
        return flist

    # Get the status of the folder
    def getFolderStatus(self, mbox, s):
        return 'Null'

    # Get the total size of the messages in the folder mbox (bytes)
    def getFolderSize(self, mbox):
        return 0

    # Get the total size of the messages in the currently selected folder (bytes)
    def getCurrentFolderSize(self):
        return 0

    # Get a list of objects with the content of the body of mail 'mnum'
    def getMessageContent(self, parent,  num):
        mlist = []
        return mlist

    # Get a dump of the content of the mail 'mnum'
    def getMessageContentRaw(self, mnum):
        text = ''
        return text

    # Get list containg the mail folders
    def getFolderList(self):
        mbs = ' '
        return string.split(mbs)
                        
    # Get a list containing the sucribed folders
    def getSubscribedFolderList(self):
        mbs = ' '
        return string.split(mbs)

    # Check if the folder is a subscribed folder
    def checkFolderSubscription(self,s):
        return None

    # Subscribe on folder s
    def subscribeFolder(self, s):
        pass

    # Cancel subscription on folder s
    def unsubscribeFolder(self, s):
        pass

    # Get 'from' header field in mail mnum 
    def getFromAddress(self, mnum):
        from_ = ''
        return from_

    # Return the To-address in the mail mnum
    def getToAddress(self, mnum):
        return ''

    # Return the Cc-address in the mail mnum
    def getCcAddress(self, mnum):
        return ''

    # Get 'subject' header field in mail mnum
    def getSubject(self, mnum):
        subj_ = ''
        return subj_

    # Return the date of the message as a date tuple and a formatted string
    def getDate(self, mnum):
        return [], ''

    # Return the size of the message
    def getSize(self, mnum):
        return ''

    # Move message mnum in current selected mbox to toBox
    def moveMessage(self, mnum, toBox):
        pass

    # Delete message mnum
    def deleteMessage(self, mnum):
        pass

    # Save a list of messages in the toBox
    def saveMessage(self, toBox, messages):
        pass

    # Create folder with name
    def createFolder(self, name):
        pass
        
    # Rename folder with name 'old' to name 'new'
    def renameFolder(self, old, new):
        result =  ['OK']
        return result

    # Delete folder identified by id 
    def deleteFolder(self, id):
        pass

    # Delete all messages in a selected folder
    def purgeFolder(self, s):
        pass

############################################################################
#
############################################################################

#class ImapServer(Pmw.MegaWidget):
class ImapServer(NullMailServer):
    """
    IMAP server abstraction
    """

    def __init__(self, parent, **kw):

        optiondefs = (
           ('config', None, Pmw.INITOPT),
           ('confdir', None, Pmw.INITOPT),
            )

        self.defineoptions(kw, optiondefs)
        NullMailServer.__init__(self, parent)

        self.M = None
        self.status = 'DOWN'
        self.parent = parent
        self.mboxSize = 0
        self.recentList = []

        self.readConf()
        
        if self.login() != 'UP':
            return

        # Get hierarchy separator from server
        try:
            s =  self.M.list(self.maildir, '')
            if s[0] == 'OK':
                mo = re.match('\(.*\) \"(.+)\" .*', s[1][0])
                if mo != None:
                    self.separator = mo.groups()[0]
                else:
                    print "Failed to retrieve IMAP hierarchy separator"
                    self.separator = '/'
        except IndexError:
                    print "Failed IMAP hierarchy separator"
                    self.separator = '/'


    def __del__(self):
        if self.status == 'UP':
            try:
                self.M.close()
            except imaplib.IMAP4.error: pass

    def readConf(self):
        # Read IMAP config
        try:
            self.host =  conf.get("IMAP", "host")
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            self.host = 'localhost'
            #print "Default IMAP host is localhost"
            if hasattr(conf, 'set'):
                conf.set('IMAP', 'host', self.host)

        try:
            self.port =  int(conf.get("IMAP", "port"))
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            self.port = 143
            #print "Default IMAP port is 143"
            if hasattr(conf, 'set'):
                conf.set('IMAP', 'port', self.port)            

        try:
            self.user =  conf.get("IMAP", "user")
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            self.user = getpass.getuser()
            #print "Default IMAP user name same as system login: ", user
            if hasattr(conf, 'set'):
                conf.set('IMAP', 'user', self.user)            

        try:
            self.inbox =  self['config'].get("IMAP", "inbox")
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            self.inbox = 'INBOX'
            #print 'Default inbox:', self.inbox
            if hasattr(conf, 'set'):
                conf.set('IMAP', 'inbox', self.inbox)            

        try:
            self.maildir =  self['config'].get("IMAP", "maildir")
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            #print "Default maildir: ~/mail"
            self.maildir='~/mail/'
            if hasattr(conf, 'set'):
                conf.set('IMAP', 'maildir', self.maildir)            

        try:
            fn = self['confdir'] + 'mamamua/password'
            fp = open(fn, 'r')
            import rotor
            rt = rotor.newrotor(self.user)
            self.passw = rt.decrypt(pickle.load(fp))
            fp.close()
        except IOError, KeyError:
            self.passw = None

        self.expires = {}
        self.maxsizes = {}
        l = self['config'].sections()
        for s in l:
            mo = re.match('IMAPFOLDER_(.+)', s)
            if mo != None:
                try:
                    n = self['config'].get(s, "expire")
                    if n != '-':
                        self.expires[mo.groups()[0]] = n
                except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
                    self['config'].set(s, 'expire', '-')
                try:
                    n =  self['config'].get(s, "maxsize")
                    if n != '-':
                        self.maxsizes[mo.groups()[0]] = n
                except(ConfigParser.NoOptionError, ConfigParser.NoSectionError):
                    if hasattr(self['config'], 'set'):
                        self['config'].set(s, 'maxsize', '-')

                
    def login(self):
        # IMAP login
        if self.M == None:
            try:
                self.M = imaplib.IMAP4(self.host, self.port)
            except socket.error:
                print 'IMAP socket error', self.host, self.port
                return self.status

        if self.passw == None:
            saveFlag = IntVar()
            pw = PasswordDialog(root, saveFlag)
            curpass = pw.get()
            del pw
            if saveFlag.get():
                try:
                    fn = self['confdir'] + 'mamamua/password'
                    fp = open(fn, 'w')
                    import rotor
                    rt = rotor.newrotor(self.user)
                    pickle.dump(rt.encrypt(curpass), fp)
                    fp.close()
                except IOError, KeyError:
                    print 'Can not save password'

        else:
            curpass = self.passw

        if curpass != '':
            try:
                self.M.login(self.user, curpass)
                self.status = 'UP'
            except imaplib.IMAP4.error:
                self.errorMessage('IMAP login failed')
                os.remove(self['confdir'] + 'mamamua/password')

        return self.status


    def logout(self):
        if self.M != None:
            self.M.close()


    def reopen(self):
        del self.M
        self.M = None
        self.login()


    def getStatus(self):
        return self.status


    def getFolderExpire(self, m):
        s = re.sub('/', '_SLASH_', m)
        s = re.sub('~', '_TILDE_', s)
        try:
            n = int(self.expires[s])
            e = time.time() - n*24*3600
        except  KeyError:
            e = 0
        return e

    def getFolderOption(self, m, opt):
        try:
            s = re.sub('/', '_SLASH_', m)
            s = re.sub('~', '_TILDE_', s)
            s = 'IMAPFOLDER_' + s
        except TypeError:
            return None
        
        try:
            val = self['config'].get(s, opt)
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            val = None
            
        if val == '-':
            val = None
                
        return val

    def getHierachySeparator(self):
        return self.separator


    def getFolderContent(self, mbox):
        #print 'getfolder content', mbox
        flist = []
        if mbox == None:
            mbox = (self.inbox,)

        if mbox != None:
            if ((self.M.select(mbox[0]))[0] == 'OK'):
                self.mboxSize = 0
                self.mbox = mbox[0]
                typ, data = self.M.search(None, 'ALL')

                current_expire = self.getFolderExpire(self.mbox)
                if current_expire == None:
                    current_expire = 0
                remlist = []

                for num in string.split(data[0]):
                    typ, uid = self.M.fetch(num, 'UID')
                    mo = re.match( '.*\(UID (.*)\).*', uid[0])
                    if mo != None:
                        uid = mo.groups()[0]
                    else:
                        uid = None

#                    # bodysturcture
#                    t, s = self.M.uid('fetch', uid, '(BODYSTRUCTURE)')
#                    print "Bodystructure", t, s

#                    typ, str = self.M.uid('fetch', uid, 
#                        '(BODY[HEADER.FIELDS (SUBJECT)] \
#                          BODY[HEADER.FIELDS (FROM)] \
#                          BODY[HEADER.FIELDS (DATE)] \
#                          FLAGS RFC822.SIZE)')
#
#                    print 'FIELDS'
#                    for s in str:
#                        print '\t', s

                    typ, subj = self.M.uid('fetch', uid, '(BODY[HEADER.FIELDS (SUBJECT)])')
                    typ, from_ = self.M.uid('fetch', uid, '(BODY[HEADER.FIELDS (FROM)])')
                    typ, flags = self.M.uid('fetch', uid, '(FLAGS)')
                    typ, size_ = self.M.uid('fetch', uid, '(RFC822.SIZE)')

                    self.mboxSize = self.mboxSize + int( t_rfc822size(size_))

                    subj = re.sub( '\n|\r','',t_unmime(subj[0][1]) )
                    subj = re.sub( 'Subject: ', '', subj)

                    from_ = re.sub( '\n|\r','',t_unmime(from_[0][1]) )
                    from_ = re.sub( 'From: ', '', from_)

                    flags_ = imaplib.ParseFlags(flags[0])
                    if '\\Recent' in flags_:
                        from_ = '-> ' + from_
                        self.recentList = self.recentList + [self.mbox + uid]
                        #print self.recentList
                    else:
                        if (self.mbox + uid) in self.recentList:
                            from_ = '-> ' + from_

                    date_t, date = self.getDate_(uid)

                    if (time.mktime(date_t) > current_expire):
                        flist = flist + [(uid, from_, subj, date),]
                    else:
                        remlist = remlist + [uid]

                if remlist != []:
                    for u in remlist:
                        #print 'Message', u, 'in', self.mbox, 'has expired'
                        pass
                    self.deleteMessage_(remlist)

                self.M.close()

        return flist


    def getFolderStatus(self, mbox, s):
        return self.M.status(mbox, s)


    def getFolderSize(self, mbox):
        if mbox == None:
            mbox = self.inbox

        if mbox != None:
            if ((self.M.select(mbox))[0] == 'OK'):
                self.mboxSize = 0
                self.mbox = mbox
                typ, data = self.M.search(None, 'ALL')

                for num in string.split(data[0]):
                    typ, size_ = self.M.fetch(num, '(RFC822.SIZE)')
                    self.mboxSize = self.mboxSize + int( t_rfc822size(size_))

        return self.mboxSize


    def getCurrentFolderSize(self):
        return self.mboxSize


    def getMessageContent(self, parent,  num):
        self.M.select(self.mbox)
        typ, uid = self.M.uid('fetch', num, 'UID')
        mo = re.match( '.*\(UID (.*)\).*', uid[0])
        if mo != None:
            uid = mo.groups()[0]
        typ, h = self.M.uid('fetch', num, '(BODY[HEADER.FIELDS (FROM SUBJECT TO CC)])')
        pre = ''
        for s in h[0][1:]:
            s = t_unmime(re.sub('\r','',s))
            pre = pre + '\n' + s

        typ, text = self.M.uid('fetch', num, '(BODY[HEADER] BODY[TEXT])')
        stream = StringIO.StringIO(   re.sub('\r','',
                       text[0][1]) + re.sub('\r','', text[1][1])   )

#        # bodysturcture
#        t, s = self.M.uid('fetch', uid, '(BODYSTRUCTURE)')
#        print "Bodystructure", t
#        for s1 in s:
#            print s1

        mlist = [pre]
        mlist  = mlist + extract_mime(parent, stream)

        try:
            l = []
            for item in self.recentList:
                l.append(item)
            del self.recentList[ self.recentList.index(self.mbox + uid)]
            #print l
            #print self.recentList
        except ValueError:
            pass
        self.M.close()
        return mlist


    def getMessageContentRaw(self, mnum):
        self.M.select(self.mbox)
        typ, text = self.M.uid('fetch', mnum, '(BODY[HEADER] BODY[TEXT])')
        self.M.close()
        return text


    def getFolderList(self):
        mbs = ' '

        if (self.inbox != ' - '):
                typ, listData1 = self.M.list(self.inbox)
        else:
                listData1 = ''

        typ, listData2 = self.M.list(self.maildir)

        if listData1 != '':
                mboxList = listData1 + listData2
        else:
                mboxList = listData2

        for s in mboxList:
                l = string.split(s)
                m = l[len(l) -1]
                mbs =  mbs + ' ' + m

        l = string.split(mbs)
        l.sort()

        return l


    def getSubscribedFolderList(self):
        typ, mboxList = self.M.lsub()
        if mboxList == [None]: return []

        
        l = []
        for s in mboxList:
            l.append(string.split(s)[2])
        l.sort()
        return l


    def checkFolderSubscription(self,s):
        typ, s1 = self.M.lsub(s)
        status = None
        if typ == 'OK':
            try:
                l = string.split(s1[0])
                m = l[len(l) -1]
                if m == s:
                    status = m
            except AttributeError:
                pass

        return status
 

    def subscribeFolder(self, s):
        self.M.subscribe(s)


    def unsubscribeFolder(self, s):
        self.M.unsubscribe(s)


    def getFromAddress_(self, mnum):
        typ, from_ = self.M.uid('fetch', mnum, '(BODY[HEADER.FIELDS (FROM)])')
        #print 'getFromAddress_', mnum, typ, from_
        from_ = re.sub( '\n|\r','',t_unmime(from_[0][1]) )
        from_ = re.sub( 'From: ', '', from_)
        return from_


    def getFromAddress(self, mnum):
        self.M.select(self.mbox)
        from_ = self.getFromAddress_(mnum)
        self.M.close()
        return from_


    def getToAddress_(self, mnum):
        typ, to_ = self.M.uid('fetch', mnum, '(BODY[HEADER.FIELDS (TO)])')
        to_ = re.sub( '\n|\r','',t_unmime(to_[0][1]) )
        to_ = re.sub( 'To: ', '', to_)
        return to_


    def getToAddress(self, mnum):
        self.M.select(self.mbox)
        to_ = self.getToAddress_(mnum)
        self.M.close()
        return to_


    def getCcAddress_(self, mnum):
        typ, cc_ = self.M.uid('fetch', mnum, '(BODY[HEADER.FIELDS (CC)])')
        cc_ = re.sub( '\n|\r','',t_unmime(cc_[0][1]) )
        cc_ = re.sub( 'Cc: ', '', cc_)
        return cc_


    def getCcAddress(self, mnum):
        self.M.select(self.mbox)
        cc_ = self.getCcAddress_(mnum)
        self.M.close()
        return cc_


    def getSubject_(self, mnum):
        typ, subj_ = self.M.uid('fetch', mnum, '(BODY[HEADER.FIELDS (SUBJECT)])')
        subj_ = re.sub( '\n|\r','',t_unmime(subj_[0][1]) )
        subj_ = re.sub( 'Subject: ', '', subj_)
        return subj_


    def getSubject(self, mnum):
        self.M.select(self.mbox)
        subj_ = self.getSubject_(mnum)
        self.M.close()
        return subj_


    def getDate_(self, mnum):
        typ, date_ = self.M.uid('fetch', mnum, '(INTERNALDATE)')
        date_t = imaplib.Internaldate2tuple(date_[0])
        date_s = time.strftime('%Y-%m-%d, %X', date_t)
        return date_t, date_s


    def getDate(self, mnum):
        self.M.select(self.mbox)
        date_t, date_s  = self.getDate_(mnum)
        self.M.close()
        return date_t, date_s



    def getSize(self, mnum):
        self.M.select(self.mbox)
        typ, size_ = self.M.uid('fetch', mnum, '(RFC822.SIZE)')
        self.M.close()
        return t_rfc822size(size_)


    # Move message mnum in current selected mbox to toBox
    def moveMessage(self, mnums, toBox):
        self.M.select(self.mbox)
        for mnum in mnums:
            #print 'move', mnum, 'in', mnums, 'to', toBox
            n = mnum
            #print 'n', n
            self.M.uid('copy', n, toBox)
            self.deleteMessage_([n])
        self.M.close()


    def  deleteMessage_(self, mnums):
        for mnum in mnums: 
            n = mnum
            #print 'delete', n, mnum
            typ, text = self.M.uid('fetch', n, '(FLAGS)')
            #print text
            self.M.uid('store', n, 'FLAGS', '(\Deleted)')
            typ, text = self.M.uid('fetch', n, '(FLAGS)')
            expunge = self.M.expunge()
            #print 'expunge: ', expunge


    def  deleteMessage(self, mnums): 
        self.M.select(self.mbox)
        self.deleteMessage_(mnums)
        self.M.close()


    def saveMessage(self, toBox, messages):
        for message in messages:
            d = time.gmtime(time.time())
            self.M.append(toBox, '\Seen', d, message )


    def createFolder(self, name):
        self.M.create(name)


    def renameFolder(self, old, new):
        result =  self.M.rename(old, new)
        return result


    def deleteFolder(self, id):
        status = self.M.delete(id)
        if status[0] == 'NO':
            self.M.close()
            self.M.delete(id)


    def purgeFolder(self, s):
        if (self.M.select(s)[0] == 'OK'):
            typ, data = self.M.search(None, 'ALL')
            for mnum in string.split(data[0]):
                self.M.store (mnum, 'FLAGS', '(\Deleted)')
            expunge = self.M.expunge()


    def errorMessage(self, s):
        self.dialog1 = Pmw.MessageDialog(self.parent,
                                        title = 'IMAP error',
                                        defaultbutton = 0,
                                        message_text = s)
        self.dialog1.iconname('IMAP error')
        self.dialog1.activate()


############################################################################
#
############################################################################

class MessageWidget(Pmw.ScrolledText):
    def __init__(self, parent, **kw):

        self.parent = parent

        optiondefs = (
            ('mailheader', '*', None),
            ('confdir', None, Pmw.INITOPT),
                     )

        self.defineoptions(kw, optiondefs)

        Pmw.ScrolledText.__init__(self, parent)
        self.initialiseoptions(MessageWidget)

        self.bind('<ButtonRelease-3>', self.showheader)

        self.balloon = Pmw.Balloon(self.parent)
        self.balloon.configure(statuscommand = self.interior())



    def showheader(self, event=None):
        toplevel = Toplevel(self.parent)

        t = Pmw.ScrolledText(toplevel)
        t.insert(END, self['mailheader'])
        t.insert(END,'-------------------------------------\n')
        t.insert(END, self.get())
        t.pack(expand=1, fill=BOTH)


    def setSize(self, mailsize):
        self.balloon.bind(self, mailsize + ' bytes')


    def _nametowidget(self, s):
        return self.interior()._nametowidget(s)

    def clearSize(self):
        self.balloon.bind(self, _('No message selected above'))

############################################################################
#
############################################################################

class MailWidget(Pmw.MegaWidget):
    """
    This is the UI part
    """

    def __init__(self, parent, **kw):

        optiondefs = (
           ('mailserver', None, Pmw.INITOPT),
           ('config', None, Pmw.INITOPT),
           ('confdir', None, Pmw.INITOPT),
                     )
        
        self.defineoptions(kw, optiondefs)
        Pmw.MegaWidget.__init__(self, parent)

        self.server = self['mailserver']
        
        self.readConf()

        self.parent = parent
        self.paneCount = 0
        self.message = None
        self.mbox = None
        self.mbxidx = None
        self.mbxnwlab = None
        self.mailBoxAfterId = None
        self.mailBoxCancelAfterId = None
        self.unseen = 0
        self.cmdEntry = None
        self.sep = self['mailserver'].getHierachySeparator()
        self.gmtry = {}
        self.readGeometry()

        self.menubar = Menu(parent)
        self.menubar.configure(bg='lightblue')

        self.filemenu = Menu(self.menubar, tearoff = 0)
        self.filemenu.add_command(label = _('Address book'), command = self.addressbook)
        if self.trashfolder != None:
            self.filemenu.add_command(command = self.purgetrash)
            self.trashitemindex = 1
            self.updateTrashSelection()

        self.filemenu.add_command(label = _('Add command line'), command = self.cmdline)
        self.filemenu.add_separator()
        self.filemenu.add_command(label = _('Reconnect'), command = self.reopen)
        self.filemenu.add_command(label = _('Exit'), command = root.quit)
        self.menubar.add_cascade(label = _('File'), menu = self.filemenu)

        self.messagemenu = Menu(self.menubar, tearoff = 0)
        self.messagemenu.add_command(label = _('New message'), command = self.newMail)

        self.currentmessagemenu = Menu(self.messagemenu, tearoff = 1)
        self.currentmessagemenu.add_command(label = _('Move'),
                                            command = self.moveMessage)
        self.currentmessagemenu.add_command(label = _('Delete'),
                                            command = self.deleteMessage)

        self.currentmessagemenu.add_command(label = _('Reply'),
                                            command = self.replyToMessage)
        self.currentmessagemenu.add_command(label = _('Reply to all'),
                                            command = self.replyToAllMessage)
        self.currentmessagemenu.add_command(label = _('Forward'),
                                            command = self.forwardMessage)

        if self.fixedForwardAddress != None:
            self.currentmessagemenu.add_command(
               label = _('Relay to ') + self.fixedForwardAddress,
                                            command = self.predefForwardMessage)
 
        self.currentmessagemenu.add_command(label = _('Print'),
                                            command = self.printMessage)

        self.currentmessagemenu.add_command(label = _('Export'),
                                            command = self.exportMessage)

        self.currentmessagemenu.add_command(label = _('Edit'),
                                            command = self.editMessage)

        self.currentmessagemenu.add_command(label = _('Open in window'),
                                            command = self.openMessageInWindow)

        self.currentmessagemenu.add_command(label = _('Dump raw'),
                                            command = self.getMessageContentRaw)

        self.messagemenu.add_cascade(label = _('Currently selected ...'),
                                           menu = self.currentmessagemenu)
        self.menubar.add_cascade(label = _('Message'), menu = self.messagemenu)


        self.mboxmenu = Menu(self.menubar, tearoff = 1)
        self.mboxmenu.add_command(label = _('Create ...'), command = self.newMbox)
        self.mboxmenu.add_command(label = _('Rename ...'), command = self.renameMbox)
        self.mboxmenu.add_command(label = _('Delete ...'), command = self.deleteMbox)
        self.mboxmenu.add_command(label = _('Subscribe'),
                                  command = self.subscribeFolder)
        self.mboxmenu.add_command(label = _('Cancel subscription'),
                                  command = self.unsubscribeFolder)

        self.menubar.add_cascade(label = _('Mailbox'), menu = self.mboxmenu)

        if hasattr(conf, 'write'):
            self.prefmenu = Menu(self.menubar)
            self.prefmenu.add_command(label = _('Edit config values'), command = self.optionsDialog)
            self.prefmenu.add_command(label = _('Save config'), command = confSave)
            self.menubar.add_cascade(label = _('Preferences'), menu =self.prefmenu)
        
        self.helpmenu = Menu(self.menubar, tearoff = 0)
        self.helpmenu.add_command(label = _('Commands'), command = self.helpcommands, state = 'active')
        self.helpmenu.add_command(label = _('About'), command = self.about, state = 'active')        
        self.menubar.add_cascade(label = _('Help'), menu = self.helpmenu)

        parent.config(menu = self.menubar)

        # Create a main PanedWidget with top and bottom panes.
        self.pane = Pmw.PanedWidget(parent, hull_width=self.gmtry['pane_w'], hull_height=self.gmtry['pane_h'])

        self.pane.add('top', min=140)
        self.pane.add('bottom', min=10)

        # Create two panes in the top pane.
        self.topPane = Pmw.PanedWidget(self.pane.pane('top'), orient='horizontal',
                            hull_width = self.gmtry['top_w'],
                            hull_height = self.gmtry['top_h'])

        self.topPane.add('MboxPane', min = 150)
        self.topPane.add('MessagesPane', min = 200)
        self.topPane.pane('MboxPane').configure(background='lightblue')
        self.topPane.pane('MboxPane').configure(width=self.gmtry['MboxPane_w'])
        self.topPane.pane('MboxPane').configure(height=self.gmtry['MboxPane_h'])
        self.topPane.pane('MessagesPane').configure(width=self.gmtry['MessagesPane_w'])
        self.topPane.pane('MessagesPane').configure(height=self.gmtry['MessagesPane_h'])

        self.topPane.setnaturalsize()
        self.topPane.pack(expand = 1, fill='both')

        # Create a ScrolledListBox.
        self.mailBox = Pmw.ScrolledListBox(self.topPane.pane('MboxPane'),
                labelpos='nw',
                label_text=_('Mailboxes'),
                label_relief=GROOVE,
                label_borderwidth=3,
                label_justify=CENTER,
                label_anchor='n',
                listbox_height = 6,
                selectioncommand=self.mailBoxSelectionCommand,
                dblclickcommand=self.mailBoxDefCmd,
                usehullsize = 1,
                hull_width = self.gmtry['MboxPane_w'],
                hull_heigh = self.gmtry['MboxPane_h'],
                listbox_background='gray95',
                label_background='lightblue',
                labelmargin = 0
               )

        self.mailBox._listbox.bind(
                                   '<ButtonRelease-3>', self.mailBoxDefCmd)


        self.mailBox._listbox.bind('<Motion>', self.mailBoxMotionCmd)
        self.mailBox._listbox.bind('<Leave>', self.mailBoxLeaveCmd)

        self.messagesBox =  CollumnList(self.topPane.pane('MessagesPane'),
                collumns = [_('Id'), _('From'), _('Subject'), _('Date')],
                collumnmins = [1, 1, 250, 100],
                collumnmaxs = [30, 800, 800, 800],
                itemclick = self.messagesBoxSelectionCommand,
                itemclick3 = self.messagesBoxDefCmd,
                headerclick = self.headerCmd,
                usehullsize=1,
                hull_width = self.gmtry['MessagesPane_w'],
                hull_height = self.gmtry['MessagesPane_h'],
                )

        bottomPane = Pmw.PanedWidget(self.pane.pane('bottom'), orient='vertical')
        bottomPane.add('contents', min = 150)
        bottomPane.add('commandline')

        self.contentBox = MessageWidget(self.pane.pane('bottom'),
            borderframe = 1,
            labelpos = 'n',
            label_text=_('Messsage content'),
            usehullsize = 1,
            hull_width = self.gmtry['bottom_w'],
            hull_height = self.gmtry['bottom_h'],
            text_padx = 10,
            text_pady = 10,
            text_wrap='word',
            text_bg='grey90'
            )

        self.mailBox.configure(hscrollmode = 'dynamic')
        self.mailBox.configure(vscrollmode = 'dynamic')

        for s in self['mailserver'].getSubscribedFolderList():
            l = string.split(s)
            m = l[len(l) -1]
            self.mailBox.insert(END, m)

        self.mailBox.pack(expand = 1, fill = 'both', padx = 0, pady = 0)
        self.contentBox.pack(expand = 1, fill = 'both', padx = 0, pady = 0)
        self.pane.pack(expand = 1, fill = 'both')


        #Check trashfolder
        self.checkTrashFolder()

        # Do first poll before first display 
        self.alertActive = 0
        self.pollMbox(0)

        # Display default mbox
        self.mailBox.select_set(0)
        self.mailBoxSelectionCommand()

        self.mailBoxNum = StringVar()


    def readConf(self):
        try:
            self.fixedForwardAddress =  conf.get("MAIN", "forwardaddress")
            if self.fixedForwardAddress == '-':
                self.fixedForwardAddress = None
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            self.fixedForwardAddress = None
            print "No default fixed forward address "
            if hasattr(conf, 'set'):
                conf.set('MAIN', 'forwardaddress', '-')            

        try:
            self.trashfolder =  conf.get("MAIN", "trashfolder")
            if self.trashfolder == '-':
                self.trashfolder = None
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            self.trashfolder = None
            print "No trashfolder"
            if hasattr(conf, 'set'):
                conf.set('MAIN', 'trashfolder', '-')            


        self.trashmaxsize = self.server.getFolderOption(self.trashfolder, "maxsize")
        if self.trashmaxsize == None:
            print "No max trashfolder size"
        else:
            #print "Max trashfolder size", self.trashmaxsize
            pass
#        try:
#            self.trashmaxsize =  conf.get("MAIN", "trashmaxsize")
#        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
#            self.trashmaxsize = None
#            print "No max trashfolder size"

        try:
            self.polldelay =  int(conf.get("MAIN", "mailpollinterval"))
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            self.polldelay = 15
            if hasattr(conf, 'set'):
                conf.set('MAIN', 'mailpollinterval', self.polldelay)
            print 'Default poll delay ', self.polldelay, ' seconds'
            

        
    def saveGeometry(self):
        self.gmtry = {}
        try:
            self.gmtry['pane_w'] = self.pane.winfo_width()
            self.gmtry['pane_h'] = self.pane.winfo_height()
            self.gmtry['top_w'] = self.pane.pane('top').winfo_width()
            self.gmtry['top_h'] = self.pane.pane('top').winfo_height()
            self.gmtry['bottom_w'] = self.pane.pane('bottom').winfo_width()
            self.gmtry['bottom_h'] = self.pane.pane('bottom').winfo_height()
            self.gmtry['MboxPane_w'] = self.topPane.pane('MboxPane').winfo_width()
            self.gmtry['MboxPane_h'] = self.topPane.pane('MboxPane').winfo_height()
            self.gmtry['MessagesPane_w'] = self.topPane.pane('MessagesPane').winfo_width()
            self.gmtry['MessagesPane_h'] = self.topPane.pane('MessagesPane').winfo_height()
        except TclError:
            # Application destroyed
            return

        try:
            fn = self['confdir'] + 'mamamua/geometry'
            fp = open(fn, 'w')
            pickle.dump(self.gmtry, fp)
            fp.close()
        except IOError, KeyError:
            print 'Cannot save window geometry,', fn


    def readGeometry(self):
        try:
            fn = self['confdir'] + 'mamamua/geometry'
            fp = open(fn, 'r')
            self.gmtry = pickle.load(fp)
            fp.close()
        except IOError, KeyError:
            print 'Setting default window geometry'
            self.gmtry['pane_w'] = 800
            self.gmtry['pane_h'] = 480
            self.gmtry['top_w'] = 800
            self.gmtry['top_h'] = 220
            self.gmtry['bottom_w'] = 800
            self.gmtry['bottom_h'] = 258
            self.gmtry['MboxPane_w'] = 648
            self.gmtry['MboxPane_h'] = 220
            self.gmtry['MessagesPane_w'] = 648 
            self.gmtry['MessagesPane_h'] = 220


    def reopen(self):
        self['mailserver'].reopen()


    def updateTrashSelection(self):
        if (self.trashfolder != None):
            s = _('Empty trashfolder')
            if self.trashmaxsize != None:
                a = int(self['mailserver'].getFolderSize(self.trashfolder))
                b = int(self.trashmaxsize)
                s = s + ' (' + str(int(100*a/b)) + '%)'
            self.filemenu.entryconfig(self.trashitemindex, label = s)


    def checkTrashFolder(self):
       if (self.trashfolder != None) and (self.trashmaxsize != None):
            if self['mailserver'].getFolderSize(self.trashfolder) > int(self.trashmaxsize):
                emptyFl = IntVar()
                dialog = Pmw.Dialog(self.parent,
                    title = _('Confirm'),
                    defaultbutton = 0,
                    buttons = ('Ok',),
                    )
                dialog.withdraw()
                dialog.iconname('Confirm dialog')
                s =   (self.trashfolder + ' ' +  _('exceeds size limit') +
                                         ' (>' + self.trashmaxsize + ')')
                delText = Label(dialog.interior(),text = s)
                delText.pack(expand=1, fill='both', padx=16, pady=16)

                permButton = Checkbutton(dialog.interior(),
                                      text=_('Empty trashbox now'),
                                      variable=emptyFl)

                permButton.pack()
                result = dialog.activate()

                if emptyFl.get() != 0:
                    self.purgetrash()


    def checkFolderMaxsize(self, mbox):
       n = self.server.getFolderOption(mbox, "maxsize")
       if n != None:
           if self['mailserver'].getFolderSize(mbox) > int(n):
                emptyFl = IntVar()
                dialog = Pmw.Dialog(self.parent,
                    title = _('Confirm'),
                    defaultbutton = 0,
                    buttons = ('Ok',),
                    )
                dialog.withdraw()
                dialog.iconname('Confirm dialog')
                s =   (mbox + ' ' +  _('exceeds size limit') +
                                     ' (>' + n + ')')
                delText = Label(dialog.interior(),text = s)
                delText.pack(expand=1, fill='both', padx=16, pady=16)

#               Don't purge ordinary folder, just warn!
#                permButton = Checkbutton(dialog.interior(),
#                                  text=_('Empty') + ' ' + mbox + ' ' + _('now'),
#                                  variable=emptyFl)
#
#                permButton.pack()
                result = dialog.activate()

#                if emptyFl.get() != 0:
#                    print 'purge', mbox
#                    self['mailserver'].purgeFolder(mbox)
#                    self.mailBoxSelectionCommand()

    def pollMbox(self, MboxSelectionFl = 1):
        alertFlag = 0
        n_str = self._getUnseen('INBOX')
        if len(n_str) > 0:
            n = int(n_str)
        else:
            n = 0

        if (n > self.unseen):
            alertFlag = 1
        self.unseen = int(n)

        if (self.mbox != None):
            n = self._getUnseen(self.mbox[0])
            if len(n) > 0:
                if int(n) > 0:
                    alertFlag = 1

        if alertFlag != 0:
            self.parent.bell()
            s = _('New mail has arrived')
            s = s + '\n' + n_str

            rootstate = root.state()

            if rootstate == 'iconic':
                b_t = (_('Ok'), _('Iconify'))
            else:
                b_t = (_('Ok'),)
            d = Pmw.MessageDialog(self.parent,
                                title = _('New mail'),
                                defaultbutton = 0,
                                buttons = b_t,
                                message_text =  s
                              )
            d.iconname('New mail')
            root.deiconify()

            result = d.activate()
            del d

            if MboxSelectionFl:
                self.messagesBox.delete(0, END)
                self.contentBox.delete(0.0, END)
                self.mailBoxSelectionCommand()
                self.messagesBox.update_idletasks()

            if (rootstate == 'iconic') and (result == _('Iconify')):
                 root.iconify()

        self.pollMboxId = self.parent.after(self.polldelay*1000, self.pollMbox)

     
    def _getUnseen(self, mbox):
        s = self['mailserver'].getFolderStatus(mbox, '(UNSEEN)')
        if s[0] == 'OK':
            try:
                n = re.sub( '\)', '', re.split(' ', s[1][0])[2] )
            except IndexError:
                n = '0'
        else:
            n = ''
        return n


    def _getRecent(self, mbox):
        s = self['mailserver'].getFolderStatus(mbox, '(RECENT)')
        if s[0] == 'OK':
            n = re.sub( '\)', '', re.split(' ', s[1][0])[2] )
        else:
            n = ''
        return n


    def _getNOFMessages(self, mbox):
        s = self['mailserver'].getFolderStatus(mbox, '(MESSAGES)')
        if s[0] == 'OK':
            n = re.sub( '\)', '', re.split(' ', s[1][0])[2] )
        else:
            n = ''
        return n


    def mailBoxMotionCmd(self, event=None):
        if self.mbxnwlab == None:
            self.mbxnwlab = Listbox(self.parent)
            maxn = 0
            #print 'yview: ', event.widget.yview()
            ifirst = int(round(event.widget.yview()[0] * (event.widget.index(END) + 1)))
            inum_ =  event.widget.yview()[1] * (event.widget.index(END))
            inum = int(round(inum_))
            #print inum, inum_
            #print ifirst, inum
            for i in range (ifirst, inum):
                mbx = self.mailBox.get(i)
                nnew = self._getUnseen(mbx)
                etime = self['mailserver'].getFolderExpire(mbx)
                if etime > 0:
                    ndays = int((etime - time.time())/(24*3600))
                    s1 =   ' ' + str(ndays) + _('d')
                else:
                    s1 = ''
                str2 = '(' + re.sub(' ', '0', string.rjust(nnew, 2)) +')'
                str1 = re.sub(' ', '0',string.rjust(self._getNOFMessages(mbx), 3))
                str3 = string.rjust(s1, 5)
                s = str1 + '\t' + str2 + str3
                s = string.expandtabs(s, 4)
                n = len(s)
                if n > maxn:
                    maxn = n
                self.mbxnwlab.insert(END, s)
            self.mbxnwlab.insert(END, ' ')
            self.mbxnwlab.insert(END, ' ')
            self.mbxnwlab.insert(END, ' ')
            self.mbxnwlab.insert(END, ' ')

            self.mbxnwlab.configure(#font=('Courier', 12, 'italic'),
                                    bg='gray95',
                                    borderwidth=5,
                                    relief=RIDGE,
                                    width=maxn - 1,
                                    height=inum - ifirst)

            self.mailBoxAfterId = self.parent.after(1*100, self.mailBoxMotionCmdActivate)

        if self.mailBoxCancelAfterId != None:
            self.parent.after_cancel(self.mailBoxCancelAfterId)
        self.mailBoxCancelAfterId = self.parent.after(5*1000, self.mailBoxLeaveCmd)


    def mailBoxMotionCmdActivate(self):
        if self.mbxnwlab != None:
            self.mbxnwlab.place(
              x=self.mailBox._listbox.winfo_x() + self.mailBox._listbox.winfo_width() - 2,
              y=self.mailBox._listbox.winfo_y())
        self.mailBoxAfterId = None


    def mailBoxLeaveCmd(self, event=None):
        if self.mbxnwlab != None:
            self.mbxnwlab.destroy()
            self.mbxnwlab = None
        if self.mailBoxAfterId != None:
            self.parent.after_cancel(self.mailBoxAfterId)
            self.mailBoxAfterId = None
        self.mbxidx = None


    def mailBoxSelectionCommand(self):
        if len(self.mailBox.getcurselection()) != 0:
            self.mbox = self.mailBox.getcurselection()
            
        self.messagesBox.delete(0, END)
        flist = self['mailserver'].getFolderContent(self.mbox)
        for f in flist:
            self.messagesBox.insert(END, f)

        self.messagesBox.configure(labeltext=self.mbox)

        self.contentBox.delete(0.0, END)
        self.contentBox.clearSize()
        try:
            self.contentBox.configure(label_text=_('No message selected above'))
        except TclError:
            pass          

        self.messagesBox.yview(END)
        self.message = None
        self.contentBox.update_idletasks()


    def messagesBoxSelectionCommand(self, m, idx):
        self.contentBox.delete(0.0, END)
        self.contentBox.clearSize()

        if idx != None:
            self.message = []
            for i in idx:
                 self.message.append(self.messagesBox.get(i))
            self.contentBox.insert(END, _('Hmtar meddelande ...') )
            self.contentBox.configure(label_text='')
        else:
             self.message = self.messagesBox.getcurselection()

        if len(self.message) != 0:
            self.contentBox.delete(0.0, END)

            mlist = self['mailserver'].getMessageContent(
                         self.parent, 
                         self.message[0][0])

            for o in mlist[1:]:
                if type(o) == types.StringType:
                    self.contentBox.insert(END, o)
                elif type(o) == types.InstanceType:
                    self.contentBox.window_create(END, window = o)
                else:
                    print 'UNHANDLED TYPE\n'

            self.contentBox.configure(label_text=self.messagesBox.get(idx[0])[1])
            self.contentBox.configure(mailheader=mlist[0])
            self.contentBox.setSize(self['mailserver'].getSize(self.message[0][0]))
            del mlist
            l = []
            for m in self.message:
                l = l + [m[0]]
            if (self.message[0][1][0:2] == '->' ):
                i = self.messagesBox.curselection()
                s = self.message[0][1]
                s = s[3:]
                self.message[0][1] = s
                self.messagesBox.delete(i)
                self.messagesBox.insert(i, self.message[0])
        else:
            pass


    def mailBoxDefCmd(self, event=None):
        if event != None:
            event.widget.select_clear(0,END)
            event.widget.select_set(event.widget.nearest(event.y))

        sels = self.mailBox.getcurselection()
        if len(sels) == 0:
            self.mbox = None
        else:
            self.mbox = sels[0]

        try:
            self.grab_set()
        
            self.mboxmenu.post(self.mailBox.winfo_rootx() + 10,
                                     self.mailBox.winfo_rooty() + 20)
            self.grab_release()
        except TclError:
            pass

    def messagesBoxDefCmd(self):
        try:
            s = self.messagesBox.getcurselection()
        except IndexError:
            s = []

        if len(s) == 0:
            pass
        else:
            self.messageDialog(s)


    def headerCmd(self):
        self.messagesBox.sort(self.messagesBox.getCurrentCol())


    def insertMailboxList(self, listData):
        for s in listData:
            l = string.split(s)
            m = l[len(l) -1]
            self.mailBox.insert(END, m)
        self.mailBox.pack()


    def confirmDialog(self,s):
        dialog = Pmw.MessageDialog(self.parent,
            title = _('Confirm'),
            defaultbutton = 0,
            buttons = (_('Ok'), _('Cancel')),
            message_text = s,
            )
        dialog.iconname('Confirm dialog')
        result = dialog.activate()
        return (result == _('Ok'))


    def confirmDeleteDialog(self,s):
        permFl = IntVar()
        self.dialog = Pmw.Dialog(self.parent,
            title = _('Confirm'),
            defaultbutton = 0,
            buttons = (_('Ok'), _('Cancel')),
            )
        self.dialog.withdraw()
        self.dialog.iconname('Confirm dialog')
        delText = Label(self.dialog.interior(),text = s)
        delText.pack(expand=1, fill='both', padx=16, pady=16)

        permButton = Checkbutton(self.dialog.interior(),
                                      text=_('Permanent delete'),
                                      variable=permFl)

        permButton.pack()
        result = self.dialog.activate()

        if result == 'Ok':
            if permFl.get() == 0:
                status = 1
            else:
                status = 2
        else:
            status = 0

        return status 


    # Menu commands
    def login(self):
        self['mailserver'].open()


    def addressbook(self):
        if self.message != None:
            s = self['mailserver'].getFromAddress(self.message[0])
            a = AddressBook(self.parent,
                         selection = s,
                         confdir=self['confdir'])
        else:
            a = AddressBook(self.parent, confdir=self['confdir'])


    def cmdline(self):
        if self.cmdEntry == None:
            self.cmdEntry = Entry(self.parent)
            self.cmdEntry.bind('<Return>', self.cmdExecute)
            self.cmdEntry.bind('<Tab>', self.cmdComplete)
            p = self.mbox
            if p == None:
               p = '[None]:'
            else:
               p = '[' + p[0] + ']: '
            self.cmdEntry.insert(END, p)
            self.pane.pack(expand = 1, fill = 'both')
            self.cmdEntry.pack(expand = 1, fill = 'both', padx = 0, pady = 0)


    def cmdComplete(self, event):
        print 'cmd complete', event.widget.get()
        self.cmdEntry.focus_force()           # don't let focus go with the tab


    def cmdExecute(self, event):
        cmd = re.split(' ', event.widget.get())
        i = string.find(cmd[0], ']:')
        if i > 0 :
            s = cmd[0]
            cmd[0] = s[i+2:]
            if cmd[0] == '':
                del cmd[0]
        print 'cmd', cmd, i
        self.cmddict = {'close': self._cmd_close,
                   'exit': self._cmd_exit,
                   'mkdir': self._cmd_mkfolder,
                   'rmdir': self._cmd_rmfolder,
                   'mvdir': self._cmd_renamefolder,
                   'cd': self._cmd_selfolder,
                   'find': self._cmd_selmail,
                   'rm': self._cmd_deletemail,
                   'mv': self._cmd_movemail,
                   'nounseen': self._cmd_getNumberOfUnseen,
                   'help': self._cmd_help,
                  }

        try:
            event.widget.delete(0, END)
            self.cmddict[cmd[0]](event, cmd)
            p = self.mbox
            if p == None:
                p = '[None]:'
            else:
                p = '[' + p[0] + ']: '
            event.widget.insert(END, p)

        except KeyError:
            event.widget.delete(0, END)
            p = self.mbox
            if p == None:
                p = '[None]:'
            else:
                p = '[' + p[0] + ']: '
            event.widget.insert(END, p + 'Unknown command')


    def _cmd_exit(self, event, command):
        root.quit()


    def _cmd_close(self, event, command):
        event.widget.destroy()
        self.cmdEntry = None


    def _cmd_help(self, event, command):
            event.widget.delete(0, END)
            l = self.cmddict.keys()
            l.sort()
            event.widget.insert(END, l)


    def _cmd_mkfolder(self, event, cmd):        
        self['mailserver'].createFolder(cmd[1])
        mboxList = self['mailserver'].getSubscribedFolderList()
        self.mailBox.delete(0, END)
        self.insertMailboxList(mboxList)


    def _cmd_rmfolder(self, event, cmd):
        self['mailserver'].deleteFolder(cmd[1])
        mboxList = self['mailserver'].getSubscribedFolderList()
        self.mailBox.delete(0, END)
        self.insertMailboxList(mboxList)


    def _cmd_selfolder(self, event, cmd):
        if cmd[1] == '..':
            s =  self.mailBox.getcurselection()[0]
            i = string.rfind(s, '/')
            if i > 0:
                s = s[0:i]
        else:
            print 'cmd[1]', cmd[1]
            mo = re.match('\.\.\/.+', cmd[1])
            if mo != None:
                s1 =  self.mailBox.getcurselection()[0]
                print 's1', s1
                i = string.rfind(s1, '\/')
                print 'i', i
                if i > 0:
                    s1 = s1[0:i]
                    s = cmd[1]
                    s = s[2:]
                    s = s1 + s
                else:
                    s = s1
            else:
                s = cmd[1]

        print 's', s

        self.mailBox.select_clear(0,END)
        l = self.mailBox.get(0, END)

        found = 0
        for i in range(0, len(l)):
            if l[i] == s:
                found = 1
                break 

        if found == 1:
            self.mailBox.select_set(i)
            self.mailBoxSelectionCommand()


    def _cmd_selmail(self, event, cmd):
        n = self.messagesBox.find(0, cmd[1])
        if n != None:
            self.messagesBox.select_set(n)
            self.messagesBoxSelectionCommand(None, [(n),])


    def _cmd_renamefolder(self, event, cmd):
        self['mailserver'].renameFolder(cmd[1], cmd[2])
        mboxList = self['mailserver'].getSubscribedFolderList()
        self.mailBox.select_clear(0,END)
        self.mailBox.delete(0, END)
        self.insertMailboxList(mboxList)


    def _cmd_deletemail(self, event, cmd):
        self['mailserver'].deleteMessage(cmd[1])
        self.messageDialogEpilog()


    def _cmd_movemail(self, event, cmd):
        self['mailserver'].moveMessage(cmd[1], cmd[2])
        self.messageDialogEpilog()


    def _cmd_getNumberOfUnseen(self, event, cmd):
        s = self['mailserver'].getFolderStatus(cmd[1], '(UNSEEN)')
        print s


    def purgetrash(self):
        if self.trashfolder != None:
            self['mailserver'].purgeFolder(self.trashfolder)
        self.mailBoxSelectionCommand()


    def newMail(self):
        nm = ComposeMail(self.parent, mailserver=self.server, confdir=self['confdir'])


    def newMbox(self):
        nm = CreateMboxWidget(self.parent, self['mailserver'])
        mboxList = self['mailserver'].getSubscribedFolderList()
        self.mailBox.delete(0, END)
        self.insertMailboxList(mboxList)


    def renameMbox(self):
        nm = RenameMboxWidget(self.parent, self['mailserver'], self.mbox)
        mboxList = self['mailserver'].getSubscribedFolderList()
        self.mailBox.delete(0, END)
        self.insertMailboxList(mboxList)


    def deleteMbox(self):
        nm = DeleteMboxWidget(self.parent, self['mailserver'], self.mbox)
        mboxList = self['mailserver'].getSubscribedFolderList()
        self.mailBox.delete(0, END)
        self.insertMailboxList(mboxList)


    def subscribeFolder(self):
        mboxList = self['mailserver'].getFolderList()
        nm = SubscribeFolderWidget(self.parent, mboxList, self['mailserver'])
        mboxList = self['mailserver'].getSubscribedFolderList()
        self.mailBox.delete(0, END)
        self.insertMailboxList(mboxList)


    def unsubscribeFolder(self):
        nm = UnsubscribeFolderWidget(self.parent, self['mailserver'], None)
        mboxList = self['mailserver'].getSubscribedFolderList()
        self.mailBox.delete(0, END)
        self.insertMailboxList(mboxList)


    def about(self):
        Pmw.aboutversion(VERSION_)
        Pmw.aboutcopyright(COPYRIGHT_ + '\n' + LICENSE_)
        Pmw.aboutcontact(CONTACT_)
        aboutDialog = Pmw.AboutDialog(self.parent, applicationname = 'Mama Mua')
        aboutDialog.show()


    def helpcommands(self):
	dialog = Pmw.TextDialog(self.parent, scrolledtext_labelpos='n',
	    title = _('Mama Mua help'),
	    defaultbutton = 0,
            label_text='Commands')
        dialog.insert('end', commands_text)
	dialog.iconname(_('Mama Mua help'))
	dialog.activate()
        dialog.deactivate()

 
    def optionsDialog(self):
        nm = ConfWidget(self.parent, conf, self['mailserver'])

        
    def messageDialog(self, mnum=None):
        #print 'messageDialog mnum', mnum, self.message
        if mnum == None: mnum = self.message
        if mnum == None: return
        self.message = mnum

        self.grab_set()
        x , y = self.messagesBox.getxy()
        #print 'x ', x, ', y', y
        self.currentmessagemenu.post(self.winfo_rootx() + x, self.winfo_rooty() + y)
        self.grab_release()


    def messageDialogEpilog(self):
            self.continueflag = 1
            self.messagesBox.delete(0, END)
            self.contentBox.delete(0.0, END)
            self.mailBoxSelectionCommand()
            self.messagesBox.update_idletasks()


    def deleteMessage(self, mnum=None):
        if mnum is None: mnum = self.message
        if mnum == None: return

        if type(mnum) == types.StringType:
            mnum = [mnum, '', '', '']

        s = 'Delete message(s) '
        mnums = []
        first = 1
        for i in mnum:
            if first == 1:
                first = 0
            else:
               s = s + ', '
            s = s + i[0] 
            mnums.append(i[0])

        if self.mbox[0] == self.trashfolder:
            if self.confirmDialog(s):
                self['mailserver'].deleteMessage(mnums)
        else:
            status = self.confirmDeleteDialog(s)
            #print "Confirm status = ", status, ", trashfolder = ", self.trashfolder

            if status == 1:
                if self.trashfolder == None:
                    self['mailserver'].deleteMessage(mnums)
                else:
                    self['mailserver'].moveMessage(mnums, self.trashfolder)
            elif status == 2:
                self['mailserver'].deleteMessage(mnums)

        self.checkTrashFolder()
        self.updateTrashSelection()

        self.messageDialogEpilog()


    def moveMessage(self, mnum=None):
        if mnum is None: mnum = self.message

        mbs = ' '

        mboxList = self['mailserver'].getSubscribedFolderList()
        for s in mboxList:
                l = string.split(s)
                m = l[len(l) -1]
                mbs =  mbs + ' ' + m

        mbl = string.split(mbs)

        d = MoveMessageWidget(root, mnum, mbl, self['mailserver'])
        result = d.dialog.activate()
        del d
        
        for m in mbl:
            self.checkFolderMaxsize(m)
            
        self.messageDialogEpilog()


    def replyToMessage(self, mnums=None):
        #print 'replyToMessage mnums', mnums, self.message

        if mnums is None: mnums = self.message
        if mnums == None: return
        mnum = mnums[0]

        from_ = self['mailserver'].getFromAddress(mnum[0])  # reply to first message in selection

        to_ = self['mailserver'].getToAddress(mnum[0])
        subj_ = self['mailserver'].getSubject(mnum[0])
        date_t, date_ = self['mailserver'].getDate(mnum[0])

        s = '\n---------------------\n' + from_ + ' wrote:\n' + self.contentBox.get()
        nm = ComposeMail(root,
                         toAddress = from_,
                         toSubject = 'Re: ' + subj_,
                         extraAddress = to_,
                         messagetext=s,
                         confdir=self['confdir'],
                         mailserver = self['mailserver'])
        self.messageDialogEpilog()


    def replyToAllMessage(self, mnums=None):
        if mnums is None: mnums = self.message
        if mnums == None: return
        mnum = mnums[0]

        from_ = self['mailserver'].getFromAddress(mnum[0])
        to_ = self['mailserver'].getToAddress(mnum[0])
        toto_ = ''
        subj_ = self['mailserver'].getSubject(mnum[0])
        s = '\n---------------------\n' + from_ + ' wrote:\n' + self.contentBox.get()

        nm = ComposeMail(root,
                         toAddress = from_ + ',' + to_,
                         toSubject = 'Re: ' + subj_, 
                         messagetext=s,
                         confdir=self['confdir'],
                         mailserver = self['mailserver'])
        self.messageDialogEpilog()


    def forwardMessage(self, mnums=None, toAddress_ = None):
        if mnums is None: mnums = self.message
        if mnums == None: return
        mnum = mnums[0]

        from_ = self['mailserver'].getFromAddress(mnum[0])

        match = re.search('.*<(.+)>.*', from_)
        if match != None:
                from_ = match.groups()[0]

        subj_ = self['mailserver'].getSubject(mnum[0])
        mtext = '\n---------------------\n' + from_ + ' wrote:\n' + self.contentBox.get()

        attachment_t = self.contentBox.window_names()
        #print attachment_t
        try:
            attachment_t = map(self.contentBox._nametowidget, attachment_t)
        except TclError: pass
        #print attachment_t
        

        to_ = self['mailserver'].getToAddress(mnum[0])

        if toAddress_ == None:
            nm = ComposeMail(root,
                             toSubject = 'Fw: ' + subj_, 
                             extraAddress = to_,
                             messagetext=mtext,
                             embeddedobjects = attachment_t,
                             confdir=self['confdir'],
                             mailserver = self['mailserver'])
        else:
            nm = ComposeMail(root,
                             toSubject = 'Fw: ' + subj_,
                             extraAddress=to_, 
                             messagetext=mtext,
                             embeddedobjects = attachment_t,
                             toAddress = toAddress_,
                             confdir=self['confdir'],
                             mailserver = self['mailserver'])

        self.messageDialogEpilog()


    def predefForwardMessage(self,mnums = None):
        if mnums is None: mnums = self.message
        if mnums == None: return
        mnum = [mnums[0]]
        self.forwardMessage(mnum, self.fixedForwardAddress[0])


    def editMessage(self, mnums=None):
        if mnums is None: mnums = self.message
        if mnums == None: return
        mnum = mnums[0]

        nm = ComposeMail(root,
                         toAddress = self['mailserver'].getToAddress(mnum[0]),
                         toSubject = self['mailserver'].getSubject(mnum[0]),
                         messagetext = self.contentBox.get(),
                         confdir=self['confdir'],
                         mailserver = self['mailserver'],
                         signatureflag = 0)
        self.messageDialogEpilog()



    def printMessage(self, mnum=None):
        if mnum is None: mnum = self.message
        if mnum == None: return

        for m in mnum:
             s = self.formatMessage(m[0])
             localprint(s)

        self.messageDialogEpilog()


    def exportMessage(self, mnum=None):
        if mnum is None: mnum = self.message
        if mnum == None: return

        fn = tkFileDialog.asksaveasfilename(filetypes=[("all filez", "*")],
                                            title=_('Save message as'))
        if (fn != None) and (len(fn) != 0):
            s = ''
            for m in mnum:
                s = s + self.formatMessage(m[0])
            fp = open(fn, 'w') #os.O_WRONLY | os.O_CREAT)
            fp.write(s)
            fp.close()


    def formatMessage(self, mnum):
        from_ = self['mailserver'].getFromAddress(mnum)

        match = re.search('.*<(.+)>.*', from_)
        if match != None:
                from_ = match.groups()[0]

        subj_ = self['mailserver'].getSubject(mnum)

        to_ = self['mailserver'].getToAddress(mnum)

        toto_ = ''
        for s in re.split(',', to_):
            match = re.search('.*<(.+)>.*', s)
            if match != None:
                #print 's: ', s
                #print 'match: ', match.groups()
                toto_ = toto_ + match.groups()[0] + ';'

        cc_ = self['mailserver'].getCcAddress(mnum)

        date_t, date_ = self['mailserver'].getDate(mnum)

        s =        _('From:')    + ' ' + from_ + \
            '\n' + _('To:')      + ' ' + toto_ + \
            '\n' + _('Cc:')      + ' ' + cc_   + \
            '\n' + _('Date:')    + ' ' + date_ + \
            '\n' + _('Subject:') + ' ' + subj_ + \
            '\n---------------------\n' + \
            '\n' + self.contentBox.get()

        return s


    def openMessageInWindow(self, mnums=None):
        if mnums is None: mnums = self.message
        if mnums == None: return

        self.contentBox.showheader()
        self.messageDialogEpilog()


    def getMessageContentRaw(self, mnums=None, w=None):
        if mnums is None: mnums = self.message
        if mnums == None: return
        mnum = mnums[0]

        if w == None: w = self.contentBox

        t = self['mailserver'].getMessageContentRaw(mnum[0])
        text = ''
        for s in t:
            for s1 in s[1:]:
                text = text + s1
        text = string.replace( text, '\r', '')
        w.delete(0.0, END)
        dialog = Pmw.TextDialog(root, scrolledtext_labelpos = 'n',
                title = 'Raw dump',
                defaultbutton = 0,
                label_text = 'Raw dump of message ' + mnum[0])
        dialog.insert('end', text)
        self.messageDialogEpilog()


###########################################################################


class confirmDeleteMessageDialog(Pmw.Dialog):
    def __init__(self, parent, **kw):

        optiondefs = (
           ('permflag', None, Pmw.INITOPT),
            )

        Pmw.Dialog.__init__(self, parent)
        self.initialiseoptions(Pmw.Dialog)


############################################################################

class SelectPGPKeyWidget:
    def __init__(self, parent, outVar, recipient=None):
        self.outVar = outVar
        s =  _('----------------    Select PGP public key (recipients key)    ----------------')
        c = os.popen('gpg --list-keys')
        text = c.read()
        c.close()

        if recipient != None:
            #print recipient
            r = re.sub('\.', '\\\.', recipient)
            #print r
            rprog = re.compile('.*' + r + '.*')

        textlist = re.split('\n', text)
        prog = re.compile('pub .+')
        keys = []
        i = 0
        index = None
        for t in textlist:
            mo = prog.match(t)
            if mo != None:
                keys.append(mo.group())
                try:
                    if rprog.match(mo.group()) != None:
                        index = i
                except NameError:
                    pass
                i = i + 1

        #print 'keys', keys
        self.dialog = Pmw.SelectionDialog(parent,
            title = _('PG key selection'),
            buttons = ('Ok', 'Cancel'),
            defaultbutton = 'Ok',
            scrolledlist_labelpos = 'n',
            label_text = s,
            scrolledlist_items = keys,
            command = self.execute,
            listbox_borderwidth=0,
            listbox_selectbackground='gray95',
            listbox_selectborderwidth=2,
            listbox_background='gray85',
            listbox_relief=RAISED
            )

        if index != None:
            self.dialog.select_set(index)
        self.dialog.activate()


    def execute(self, result):
        sels = self.dialog.getcurselection()
        if result == 'Ok':
            if len(sels) == 0:
                pass
            else:
                #print result, ' You clicked on', result, ', ',  sels[0]
                prog = re.compile('pub +\S+ \S+ (.+)')
                mo = prog.match(sels[0])
                if mo != None:
                    self.outVar.set('\"' + mo.groups()[0] + '\"')
                    #print mo.groups()[0]
                else:
                     self.outVar.se('')
        else:
            pass

        self.dialog.deactivate(result)


############################################################################

class MoveMessageWidget:
    def __init__(self, parent, mnum, l, mserver):
        self.mnum = mnum
        self.l = l
        self.server = mserver

        s =  _('Move message ')

        self.mnums = []
        first = 1
        for m in mnum:
            if first == 1:
                first = 0
            else:
                s = s + ', '
            s = s + m[0] + ', '
            self.mnums.append(m[0])

        s = s + _(' to folder:')

        self.dialog = Pmw.SelectionDialog(parent,
            title = _('My SelectionDialog'),
            buttons = ('Ok', 'Cancel'),
            defaultbutton = 'Ok',
            scrolledlist_labelpos = 'n',
            label_text = s,
            scrolledlist_items = l,
            command = self.execute,
            listbox_borderwidth=0,
            listbox_selectbackground='gray95',
            listbox_selectborderwidth=2,
            listbox_background='gray85',
            listbox_relief=RAISED
            )

        self.dialog.withdraw()


    def execute(self, result):
        sels = self.dialog.getcurselection()
        if result == 'Ok':
            if len(sels) == 0:
                pass
            else:
                self.server.moveMessage(self.mnums, sels[0])
        else:
            pass

        self.dialog.deactivate(result)


############################################################################


class SubscribeFolderWidget:
    def __init__(self, parent, l, mserver):
        self.server = mserver

        self.dialog = Pmw.SelectionDialog(parent,
            title = _('My SelectionDialog'),
            buttons = ('Ok', 'Cancel'),
            defaultbutton = 'Ok',
            scrolledlist_labelpos = 'n',
            label_text = _('Subscribe to mailbox:'),
            scrolledlist_items = l,
            command = self.execute,
                    listbox_borderwidth=0,
                    listbox_selectbackground='gray95',
                    listbox_selectborderwidth=2,
                    listbox_background='gray85',
                    listbox_relief=RAISED
            )

        self.dialog.activate()


    def execute(self, result):
        sels = self.dialog.getcurselection()
        if result == 'Ok':
            if len(sels) == 0:
                print 'You clicked on', result, '(no selection)'
            else:
                self.server.subscribeFolder(sels[0])
        else:
            pass
        self.dialog.deactivate(result)


##########################################################################

class UnsubscribeFolderWidget:
    def __init__(self, parent, mserver, folder=None):

        self.server = mserver

        # Create the dialog.
        self.dialog = Pmw.Dialog(parent,
            buttons = ('OK', 'Cancel'),
            defaultbutton = 'OK',
            title = _('Unsubscribe to mailbox'),
            command = self.execute)

        self.dialog.withdraw()

        if folder == None:
            s = '~/mail/'
        else:
            s = folder

        self.entry_ = Pmw.EntryField(self.dialog.interior(),
                        labelpos = 'w',
                        label_text = _('Enter mailbox name:'),
                        value = s,
                        )

        self.entry_.pack(expand = 1, fill = 'both', padx = 0, pady = 0)
        self.result = self.dialog.activate()


    def execute(self, result):
        if  result == 'OK':
            self.server.unsubscribeFolder(self.entry_.get())

        self.dialog.deactivate(result)


##########################################################################

class CreateMboxWidget:
    def __init__(self, parent, mserver):
        self.server = mserver

        # Create the dialog.
        self.dialog = Pmw.Dialog(parent,
                buttons = ('OK', 'Cancel'),
                defaultbutton = 'OK',
                title = _('Create mailbox'),
                command = self.execute)

        self.dialog.withdraw()

        self.entry_ = Pmw.EntryField(self.dialog.interior(),
                labelpos = 'w',
                label_text = _('Enter new mailbox name:'),
                value = '~/mail/',
                )

        self.entry_.pack(expand = 1, fill = 'both', padx = 0, pady = 0)
        self.result = self.dialog.activate()


    def execute(self, result):
        if result == 'OK':
            self.server.createFolder(self.entry_.get())

        self.dialog.deactivate(result)


##########################################################################


class RenameMboxWidget:
    def __init__(self, parent, mserver, mbox = None):
        self.server = mserver

        # Create the dialog.
        self.dialog = Pmw.Dialog(parent,
                buttons = ('OK', 'Cancel'),
                defaultbutton = 'OK',
                title = _('Rename mailbox'),
                command = self.execute)

        self.dialog.withdraw()

        if mbox == None:
            s = '~/mail/'
        else:
            s = mbox

        self.entryOld = Pmw.EntryField(self.dialog.interior(),
                labelpos = 'w',
                label_text = _('Enter old mailbox name:'),
                value = s,
                )

        self.entryNew = Pmw.EntryField(self.dialog.interior(),
                labelpos = 'w',
                label_text = _('Enter new mailbox name:'),
                value = '~/mail/',
                )


        self.entryOld.pack(expand = 1, fill = 'both', padx = 0, pady = 0)
        self.entryNew.pack(expand = 1, fill = 'both', padx = 0, pady = 0)
        self.result = self.dialog.activate()


    def execute(self, result):
        if  result == 'OK':
            result = self.server.renameFolder(self.entryOld.get(),
                                              self.entryNew.get())

        self.dialog.deactivate(result)


##########################################################################


class DeleteMboxWidget:
    def __init__(self, parent, mserver, mbox=None):
        self.server = mserver

        # Create the dialog.
        self.dialog = Pmw.Dialog(parent,
            buttons = ('OK', 'Cancel'),
            defaultbutton = 'OK',
            title = _('Delete mailbox'),
            command = self.execute)

        self.dialog.withdraw()

        if mbox == None:
            s = '~/mail/'
        else:
            s = mbox

        self.entry_ = Pmw.EntryField(self.dialog.interior(),
                        labelpos = 'w',
                        label_text = _('Enter mailbox name:'),
                        value = s,
                        )

        self.entry_.pack(expand = 1, fill = 'both', padx = 0, pady = 0)
        self.result = self.dialog.activate()


    def execute(self, result):
        if  result == 'OK':
            self.server.deleteFolder(self.entry_.get())

        self.dialog.deactivate(result)


#########################################################################


def t_rfc822size(l):
    # Parse this format: l = ['1 (UID 9 RFC822.SIZE 2114)']
    match = re.search('.*RFC822.SIZE (.+)\).*', l[0])
    if match != None:
        s = match.groups()[0]
    else:
        s = None
    return s


#########################################################################
#
# Parse MIME encoded header, RFC2047
#
# ex:
#   "=?<charset>?B?<base64_coded_part>?="
#   "=?<charset>?Q?<quoted_printable_coded_part>?="
#
#########################################################################


def readnextchar(f, s):
    c = f.read(1)
    s1 = s + c
    return s1, c


def t_unmime(itext):
        inf = StringIO.StringIO(itext)
        outtext = ''

        nextchar = inf.read(1)
        while (len(nextchar) > 0):
            s1 = nextchar

            if nextchar == '=':                       # = ...
                s1, nextchar = readnextchar(inf, s1)
                if nextchar == '?':                   # =? ...
                    charset =''
                    s1, nextchar = readnextchar(inf, s1)
                    while nextchar != '?':
                         charset = charset + nextchar
                         s1, nextchar = readnextchar(inf, s1)

                                                     # =?<charset>? ...
                    s1, nextchar = readnextchar(inf, s1)

                    decoding = string.upper(nextchar)
                                                     # =?<charset>?<decoding> ...
                    s1, nextchar = readnextchar(inf, s1)

                    if nextchar == '?':
                        codedpart = ''               # =?<charset>?<decoding>? ...
                        s1, nextchar = readnextchar(inf, s1)

                        i = 0
                        while (nextchar != '?') and (i < 128):
                             codedpart = codedpart + nextchar
                             s1, nextchar = readnextchar(inf, s1)
                             i = i + 1

                                                     # =?<charset>?<decoding>?<codedpart>? ...
                        s1, nextchar = readnextchar(inf, s1)

                        if nextchar == '=':
                                                     # =?<charset>?<decoding>?<codedpart>?=
                            if decoding == 'B':
                                decoder_ = base64.decode
                            elif decoding == 'Q':
                                decoder_ = quopri.decode
                            else:
                                decoder_ = None
                                #print 'decode mime header rerror'

                            if decoder_ != None:
                                tmp1 = StringIO.StringIO(codedpart)
                                tmp2 = StringIO.StringIO()
                                decoder_(tmp1, tmp2)
                                s1 = tmp2.getvalue()
                                tmp1.close()
                                tmp2.close()
                            else:
                                s1 = s1 + nextchar

                            nextchar = inf.read(1)
                            while nextchar == ' ':
                                nextchar = inf.read(1)

            else:
                nextchar = inf.read(1)
               
            outtext = outtext + s1
                           
        return outtext 
                               

#########################################################################
#
# MIME encode header, RFC2047
# Qouted printable encoding
# ex:
#   "=?<charset>?Q?<quoted_printable_coded_part>?="
#
#########################################################################


def t_mime(itext):
    if len(itext) > 0:
        f1 = StringIO.StringIO()
        f1.write(itext)
        f1.seek(0)
        f2 = StringIO.StringIO()
        quopri.encode(f1, f2, 1 )
        f2.seek(0)
        s = f2.read()

        while s[len(s) - 1] == '\n':
            s = s[:len(s) - 1]

        if s[len(s) - 1] == '=':
            s1 = s[:len(s) - 1]
        else:
            s1 = s

        if s1 == itext:
            otext = itext
        else:
            otext = '=?iso8859-1?Q?' + s1 + '?='
    else:
        otext = ''

    return otext


def t_mimeaddr(a):
    #print 'addr', a
    mo = re.search('(..+)<(.+)>', a)
    if mo != None:
        a1 = t_mime(mo.groups()[0])
        a1 = re.sub('\n', '', a1)
        mimeaddr = a1 + ' <' + mo.groups()[1] + '>'
    else:
        mimeaddr = a

    return mimeaddr


# Parse, look for filename
def t_getfilname(itext):
    tmp1 = StringIO.StringIO(itext)
    tmp2 = StringIO.StringIO()

    mimify.unmimify(tmp1, tmp2)

    otext = tmp2.getvalue()

    prog = re.compile('.*filename=(.+)\"? *.*', re.DOTALL)
    match = prog.match(otext)

    if match == None:
        otext = " "
    else:
        otext = re.sub('\"', '', match.groups()[0])

    tmp1.close()
    tmp2.close()

    return otext
  

# Parse a message
def extract_mime(parent, stream):
    """
    Parse an RFC 822 message, and return a list of the objects in it.
    """

    msg = mimetools.Message(stream)
    msgtype = msg.gettype()
    params = msg.getplist()

    mlist = []

    if msgtype[:20] == "multipart/mixed":
        #print "*** multipart/mixed entry\n"
        file = multifile.MultiFile(stream)
                #print "*** multipart/mixed boundary: ", msg.getparam("boundary"), "\n"
        file.push(msg.getparam("boundary"))

        i = 0
        while file.next():
            i = i + 1
            #print "*** file.next ", i , "\n"
            submsg = mimetools.Message(file)
            if (submsg.gettype())[:10] == "multipart/":
                #print "*** Another boundary level found\n"
                #print "*** submsg.headers: ", submsg.headers, "\n"
                #print "*** submsg.getheader(\"Content-Type\")", \
                #    submsg.getheader("Content-Type"), "\n"

                substream = StringIO.StringIO("Content-Type: " + \
                    submsg.getheader("Content-Type") + file.read())
                #print "*** substream:\n",i
                #print  substream.getvalue(), "\n"

                mlist = mlist + extract_mime(parent, substream)

            else:
                #print "*** Submsg:\n", submsg, "\n"
                try:
                    #print "*** Sub message type: ", submsg.gettype(), "\n"
                    #print "*** Sub message encoding: ", submsg.getencoding(), "\n"
                    if submsg.gettype() == "text/plain":
                        enc = submsg.getencoding()
                        text = file.read()
                        #print "*** Text:\n", text, "\n"
                        sfile = StringIO.StringIO(text)
                        tfile = StringIO.StringIO()
                        try:
                            mimetools.decode(sfile, tfile, enc)
                            mlist = mlist + [tfile.getvalue()]
                        except ValueError:
                            mlist = mlist + [text]
                        sfile.close()
                        tfile.close()
                    else:
                        codec = quopri
                        if submsg.getencoding() == "base64":
                            codec = base64
                        if submsg.getencoding() == "quoted-printable":
                            codec = quopri

                        c_disp =  submsg.getheader("Content-Disposition")
                        if (c_disp == None):
                            c_disp = 'No Content-Disposition in header'
                        eo = Embedded_object(parent, submsg.gettype() + " - " + \
                            c_disp, file.read(), \
                            codec )
#                        eo = Embedded_object(parent, submsg.gettype() + " - " + \
#                            c_disp, "Object text", \
#                            codec )

                        mlist = mlist + [eo]


                except ValueError:
                    #print "*** ValueError\n"
                    #print submsg
                    continue
        file.pop()


    elif msgtype[:100] == "multipart/alternative":
        #print "*** multipart/alternative entry\n"
        file = multifile.MultiFile(stream)
        file.push(msg.getparam("boundary"))
        while file.next():
            submsg = mimetools.Message(file)
            try:
                if submsg.gettype() == "text/plain":
                    enc = submsg.getencoding()
                    text = file.read()
                    #print "*** Text:\n", text, "\n"
                    sfile = StringIO.StringIO(text)
                    tfile = StringIO.StringIO()
                    #print enc
                    if enc == '7bit':
                        mlist = mlist + [text]
                    else:
                        mimetools.decode(sfile, tfile, enc)
                        mlist = mlist + [tfile.getvalue()]
                    tfile.close()
                    sfile.close()
            except ValueError:
                print "*** ValueError\n"
                continue
        file.pop()

#    elif msgtype[:10] == "multipart/digest":
#        #print "*** multipart/digest entry\n"
#        file.pop()

#    elif msgtype[:10] == "multipart/encrypted":
#       #print "*** multipart/entry entry\n"
#
#       From rfc2015
#       Expect protocol="application/pgp-encrypted"
#
#       --foo
#       Content-Type: application/pgp-encrypted
#
#       Version: 1
#
#       --foo
#       Content-Type: application/octet-stream
#
#       -----BEGIN PGP MESSAGE-----
#       Version: 2.6.2
#
#       hIwDY32hYGCE8MkBA/wOu7d45aUxF4Q0RKJprD3v5Z9K1YcRJ2fve87lMlDlx4Oj
#       eW4GDd
#
#       -----END PGP MESSAGE-----
#   
#       --foo--
#
#        file = multifile.MultiFile(stream)
#        file.push(msg.getparam("boundary"))
#        file.next()
#        submsg = mimetools.Message(file)
#        if submsg == 'application/pgp-encrypted':
#            text = file.read()
#            mlist = mlist + [text]
#            file.next()
#            submsg = mimetools.Message(file)
#            if submsg == 'application/octet-stream'
#                enctext == file.read()
#                fn = tempfile.mktemp()
#                f = os.open(fn, os.O_RDWR | os.O_CREAT)
#                os.write(f, enctext)
#                os.close(f)
#                c = os.popen('gpg --decrypt ' + fn)
#                dectext = c.read()
#                c.close()
#                os.unlink(fn)
#                mlist = mlist + [dectext]
#        file.pop()

    else:
        #print "*** non multipart entry\n"
        tfile = StringIO.StringIO()
        mimify.unmimify(stream, tfile)
        t = tfile.getvalue()
        po = re.compile('-----BEGIN PGP MESSAGE-----.*-----END PGP MESSAGE-----', re.DOTALL)
        mo = po.match(t)
        if mo != None:
            mo.group()
            eo = Embedded_PGP_object(parent, mo.group() )
            mlist = mlist + [eo]
            tfile.close()
            return mlist

        po = re.compile('-----BEGIN PGP SIGNED MESSAGE-----.*' + \
                        '-----END PGP SIGNATURE-----', re.DOTALL)
        mo = po.match(t)
        if mo != None:
            mo.group()
            eo = Embedded_PGP_object(parent, mo.group() )
            mlist = mlist + [eo]
            tfile.close()
            return mlist
        else:
            po = re.compile('-----BEGIN PGP PUBLIC KEY BLOCK-----.*' + \
                            '-----END PGP PUBLIC KEY BLOCK-----', 
                            re.DOTALL)
            mo = po.match(t)
            if mo != None:
                mo.group()
                eo = Embedded_PGP_KEY_object(parent, mo.group() )
                mlist = mlist + [eo]
            else:
                #print msgtype, params
                if msgtype == "text/html":
                    import formatter
                    import htmllib
                    tf = StringIO.StringIO()
                    w = formatter.DumbWriter(tf)
                    f = formatter.AbstractFormatter(w)
                    p = htmllib.HTMLParser(f)
                    p.feed(tfile.getvalue())
                    p.close()
                    tf.seek(0)
                    mlist = mlist + [tf.read()]
                else:
                    mlist = mlist + [tfile.getvalue()]

        tfile.close()

    return mlist


#########################################################################
#
#########################################################################


def localprint(s):
    """
    Cheap print
    """

    tempfilename = tempfile.mktemp('.muapr')
    f = os.open(tempfilename, os.O_RDWR | os.O_CREAT)
    os.write(f, s)
    os.close(f)
    mo = re.search("(.*)(%f)(.*)", printcommand)

    if mo != None:
        cmd = mo.groups()[0] + tempfilename + mo.groups()[2]
    else:
        cmd = printcommand + ' ' + tempfilename

    os.system(cmd)


#########################################################################
#
#########################################################################

def getMimeCmd(fn, temp_fn=None):

    #fn1 =  re.sub('['+string.punctuation+']', '.', fn)
    #print 'getMimeCmd', fn, fn1, temp_fn
    
    if temp_fn == None:
        temp_fn = fn
    try:
        cmd = mailcap.findmatch(mailcap.getcaps(), mimetypes.guess_type(fn)[0], filename=temp_fn)[0]
    except AttributeError:
        print 'No application for file', fn, temp_fn
        cmd = ''

    return cmd 

#########################################################################
#
#########################################################################

def mimeExec(fn):
    """
    Execute command corresponding to Mime type for filename (fn)
    """

    import mailcap
    import mimetypes

    if os.name == 'nt':
        try:
            os.startfile(fn)
        except AttributeError:
            print 'This version of python has no attribute os.startfile'
    else:
        try:
            os.system(mailcap.findmatch(mailcap.getcaps(), mimetypes.guess_type(fn)[0], filename=fn)[0])
        except AttributeError:
            print 'Can not start application for file', fn
            pass

    os.unlink(fn)
    

#########################################################################
#
#########################################################################
    
def mimeGetFileExt(fn):
    # get extension from self.text_ as par to mktemp
    prog = re.compile('.*filename=.*\.(.*)\"', re.DOTALL)
    mo = prog.match(fn)
    if mo != None:
        ext = '.' + mo.groups()[0]
    else:
        ext = ''
        
    return ext
  

#########################################################################
#
#########################################################################


class PasswordDialog:
    def __init__(self, parent, saveflag=None):
        # Create the dialog to prompt for the password.
        self.dialog = Pmw.PromptDialog(parent,
            title = _('Mama Mua Password'),
            label_text = _('Enter IMAP password:'),
            entryfield_labelpos = 'n',
            entry_show = '*',
            defaultbutton = 0,
            buttons = ('OK', 'Cancel'),
            command = self.execute)

        if saveflag != None:
            saveButton = Checkbutton(self.dialog.interior(),
                                          text=_('Save password'),
                                           variable=saveflag)
            saveButton.pack()

        self.parent = parent
        self.pmins = self.parent.minsize()
        self.pmaxs = self.parent.maxsize()
        self.parent.maxsize(2,2)
        self.dialog.activate()


    def execute(self, result):
        if result is None or result == 'Cancel':
            self.dialog.deactivate(result)
            self.pw = ''
        else:
            self.pw = self.dialog.get()
            self.dialog.deactivate()


    def get(self):
        self.parent.maxsize(self.pmaxs[0], self.pmaxs[1])
        self.parent.minsize(self.pmins[0], self.pmins[1])
        return self.pw


#########################################################################
#
#########################################################################

class AddressBook(Pmw.MegaWidget):
    """
    Addressbook, (should be) db for all mail address information
    """

    def __init__(self, parent, **kw):

        optiondefs = (('exitcallback', None, Pmw.INITOPT),
                      ('selection', None, Pmw.INITOPT),
                      ('dest', '', Pmw.INITOPT),
                      ('confdir', None, Pmw.INITOPT),
                      ('mode', 'Interactive', Pmw.INITOPT),
                      ('cmd', None, Pmw.INITOPT),
                      )
        
        self.defineoptions(kw, optiondefs)
        Pmw.MegaWidget.__init__(self, parent)

        self.parent = parent
        self.address = ''
        self.cursel = None
        self.address_file = None
        self.exitcallback = self['exitcallback']
        self.db = {}
        self.signfn = StringVar()

        if self['mode'] == 'command':
            return

        self.window = Toplevel(parent)

        self.menubar = Menu(self.window)
        self.menubar.configure(background='lightblue')

        self.filemenu = Menu(self.menubar, tearoff = 0)
        self.filemenu.add_command(label = _('Open'), command = self.openbook)
        self.filemenu.add_command(label = _('Save'), command = self.savebook)
        self.filemenu.add_command(label = _('Save as'), command = self.savebookas)
        self.filemenu.add_command(label = _('Import'), command = self.importbook)
        self.filemenu.add_command(label = _('Export'), command = self.exportbook)
        self.filemenu.add_command(label = _('Dump db'), command = self.dumpdb)
        self.filemenu.add_separator()
        self.filemenu.add_command(label = _('Sort'), command = self.sortbook)
        self.filemenu.add_separator()
        self.filemenu.add_command(label = _('Quit - No select'),
                                command = self.quit_)
        if self.exitcallback != None:
            self.filemenu.add_command(label = _('Select and exit'),
                                 command = self.window.destroy)
        self.menubar.add_cascade(label = _('File'), menu = self.filemenu)

        self.addressmenu = Menu(self.menubar, tearoff = 1)
        if self.exitcallback != None:
            self.addressmenu.add_command(label = _('Insert in To-field'),
                        command = self.transferFromList)
        self.addressmenu.add_command(label = _('Pick'), command = self.sel)
        self.addressmenu.add_command(label = _('Delete from book'),
                                    command = self.delete)
        self.addressmenu.add_command(label = _('Edit'), command = self.edit)
        self.menubar.add_cascade(label = _('Address'), menu = self.addressmenu)

        self.selectionmenu = Menu(self.menubar, tearoff = 1)

        if self.exitcallback != None:
            self.selectionmenu.add_command(label = _('Transfer'),
                                command = self.transfer)

        self.selectionmenu.add_command(label = _('Clear selection'),
                            command = self.clearSelection)

        self.selectionmenu.add_command(label = _('Add to book'), command = self.add)
        self.menubar.add_cascade(label = _('Selection'), menu = self.selectionmenu)

        self.window.config(menu = self.menubar)

        self.pane = Pmw.PanedWidget(self.window) #, hull_width=80, hull_height=80)
        self.pane.add('top', min=100)
        self.pane.add('bottom', min=20)

        self.addressList = CollumnList(self.pane.pane('top'),
                    labeltext = _('Mail address book'),
                    collumns = [_('Alias'), _('Address(es)')],
                    collumnmins = [180, 180],
                    itemdblclick = self.sel,
                    itemclick = self.setcursel,
                    itemclick3 = self.popupmenu,
                    )

        self.addressWidget = Pmw.EntryField(self.pane.pane('bottom'),
            labelpos = 'w',
            label_text = self['dest'] + ' ' +_('Selection:'),
            value = '',
            validate = None,
            command = self.transfer)

        self.addressWidget._entryFieldEntry.bind(
                        '<Double-Button-1>', self.transfer)

        self.addressList.insert(END, ['empty1','empty2'])  
        self.addressWidget.pack(expand = 1, fill = 'both', padx = 0, pady = 0)

        self.addressWidget.bind("<Destroy>", self.advertise)
        self.addressWidget._entryFieldEntry.bind(
                            "<ButtonRelease-3>", self.selpopupmenu)

        self.pane.pack(fill=BOTH)

        self.openbook(self['confdir'] + 'mamamua/addressbook')

        if self['selection'] != None:
            self.addressWidget.insert(END, self['selection'])

        if self['cmd'] == 'add':
            self.add()
       
 
    def query(self, q):
        ret = None

        fn = confdir + 'mamamua/addressbook'
        try:
            fp = open(fn, 'r')
            self.db = pickle.load(fp)
            l = self.db.keys()
            l.sort()
            for s in l:
                try:
                    if self.db[s]['email'] == q:
                        ret = s + ' <' + self.db[s]['email'] + '>'
                except TypeError:
                    pass
            fp.close()
        except IOError:
            print 'Error open file', fn

        return ret


    def queryKey(self, q, val):
        ret = []

        fn = confdir + 'mamamua/addressbook'
        try:
            fp = open(fn, 'r')
            self.db = pickle.load(fp)
            l = self.db.keys()
            l.sort()
            for s in l:
                try:
                    if self.db[s][q] == val:
                        ret = ret + [s]
                except KeyError:
                    pass
            fp.close()
        except IOError:
            print 'Error open file', fn
        if ret == []:
            ret = None

        return ret


    def dumpdb(self):
        l = self.db.keys()
        l.sort()
        for s in l:
            l1 = self.db[s].keys()
            l1.sort()
            #print s
            for s1 in l1:
                print '\t', s1, '\t', self.db[s][s1]

            
    def readKey(self, s, k):
        ret = None
        fn = confdir + 'mamamua/addressbook'
        try:
            fp = open(fn, 'r')
            self.db = pickle.load(fp)
            try:
                ret = self.db[s][k]
            except KeyError:
                pass
            fp.close()
        except IOError:
            print 'Error open file', fn

        return ret


    def advertise(self, event=None):
        if self.exitcallback != None:
            self.exitcallback(self.address)


    def transfer(self, event=None):
        if self.exitcallback != None:
            self.exitcallback(self.address)
            self.addressWidget.delete(0, END)
            self.address = ''


    def transferFromList(self, event=None):
        if len(self.addressList.curselection()) != 0:
            s = "\"" + self.addressList.get(self.addressList.curselection())[0]  + "\" " + \
                "<" + self.addressList.get(self.addressList.curselection())[1] + ">"
            if self.exitcallback != None:
                self.addressWidget.delete(0, END)
                self.addressWidget.insert(END, s)
                self.exitcallback(s)
                self.addressWidget.delete(0, END)
                self.address = ''


    def quit_(self):
        self.address = ''
        self.window.destroy()


    def execute(self):
        pass


    def openbook(self, in_fn = None):
        if in_fn == None:
            fn = tkFileDialog.askopenfilename(filetypes=[("all filez", "*")])
            if len(fn) != 0:
                self.address_file = fn
        else:
            self.address_file = in_fn

        self.addressList.delete(0, END)

        try:
            fp = open(self.address_file, 'r')
            self.db = pickle.load(fp)
            l = self.db.keys()
            l.sort()
            for s in l:
                self.addressList.insert(END, [s,self.db[s]['email']])
        except IOError:
            fp = open(self.address_file, 'w')

        fp.close()


    def importbook(self):
        fn = tkFileDialog.askopenfilename(filetypes=[("all filez", "*")])
        if len(fn) != 0:
            fp = open(fn, 'r')
        else:
            return

        self.addressList.delete(0, END)

        for s in string.split(fp.read(), '\n'):
            if len(s) != 0:
                sp = string.split(s, ';')
                self.addressList.insert(END, sp)
                try:
                    self.db[sp[0]]['email'] = sp[1]
                except KeyError:
                    self.db[sp[0]] = {}
                    self.db[sp[0]]['email'] = sp[1]

        self.addressList.sort(0)
        fp.close()


    def savebook(self):
        if self.address_file == None:
            self.savebookas()
        else:
            self.savebookas(self.address_file)


    def savebookas(self, in_fn = None):
        if in_fn == None:
            fn = tkFileDialog.asksaveasfilename(
                            filetypes=[("all filez", "*")],
                            title=_('Save addressbook as'))
            self.address_file = fn
        else:
            self.address_file = in_fn

        fp = open(self.address_file, 'w') #os.O_WRONLY | os.O_CREAT)
        for s in self.addressList.get(0,END):
            try:
                self.db[s[0]]['email'] = s[1]
            except KeyError:
                self.db[s[0]] = {}
                self.db[s[0]]['email'] = s[1]

        self.dumpdb
        pickle.dump(self.db, fp)
        fp.close()


    def exportbook(self):
        fn = tkFileDialog.asksaveasfilename(
                        filetypes=[("all filez", "*")],
                        title=_('Export addressbook as'))

        fp = open(fn, 'w')
        for s in self.addressList.get(0,END):
             fp.write(s[0] + ';' + s[1] + '\n')
        fp.close()


    def popupmenu(self):
        self.addressmenu.grab_set()
        x , y = self.addressList.getxy()
        self.addressmenu.post(self.window.winfo_rootx() + x,
                             self.window.winfo_rootx() + y)
        self.addressmenu.grab_release()


    def selpopupmenu(self, event=None):
        self.selectionmenu.grab_set()
        x , y = self.addressList.getxy()
        self.selectionmenu.post(self.addressWidget.winfo_rootx()+ 10,
                     self.addressWidget.winfo_rooty() + 20)
        self.selectionmenu.grab_release()


    def sortbook(self):
        self.addressList.sort(0)


    def sel(self, event=None):
        if len (self.addressList.curselection()) != 0:
            if len(self.addressWidget.get()) != 0:
                self.addressWidget.insert(END, ', ')
            try:
                string.index(self.addressList.getcurselection()[0][1], ',')
                # It contains at least one ',', it is an address list (group)
                self.addressWidget.insert(END,
                    self.addressList.getcurselection()[0][0] + ': ')
                self.addressWidget.insert(END,
                    self.addressList.getcurselection()[0][1] + ';')
            except ValueError:
                # String contains no ',', it is not an address list
                self.addressWidget.insert(END,
                    self.addressList.getcurselection()[0][0])
                self.addressWidget.insert(END,
                    ' <' + self.addressList.getcurselection()[0][1] + '>')
  
            self.cursel = self.addressList.curselection()
        else:
            self.cursel = None

        self.address = self.addressWidget.get()


    def clearSelection(self):
        self.addressWidget.delete(0, END)


    def setcursel(self, m = None, event=None):
        if len (self.addressList.curselection()) != 0:
            self.cursel = self.addressList.curselection()
        else:
            self.cursel = None


    def add(self):
        if len(self.addressWidget.get()) != 0:
            # Create the dialog.
            self.dialog = Pmw.Dialog(self.parent,
                buttons = ('OK', 'Cancel'),
                defaultbutton = 'OK',
                title = _('Create new address entry'),
                command = self.add_execute)

            self.dialog.withdraw()

            self.aliasentry = Pmw.EntryField(self.dialog.interior(),
                labelpos = 'w',
                label_text = _('Alias:'),
                value = 'New',
                )

            self.aliasentry.pack(expand = 1, fill = 'both', padx = 0, pady = 0)

            self.addressentry = Pmw.EntryField(self.dialog.interior(),
                labelpos = 'w',
                label_text = _('Address:'),
                value = self.addressWidget.get(),
                )

            self.addressentry.pack(expand = 1, fill = 'both', padx = 0, pady = 0)

            self.result = self.dialog.activate()


    def add_execute(self, result):
        if  result == 'OK':
            self.addressList.insert(END, [self.aliasentry.get(), self.addressentry.get()])

        self.dialog.deactivate(result)


    def delete(self):
        if len(self.addressList.curselection()) != 0:
            self.addressList.delete(self.addressList.curselection())


    def edit(self):
        if self.cursel != None:
            # Create the dialog.
            self.dialog = Pmw.Dialog(self.parent,
                buttons = ('OK', 'Cancel'),
                title = _('Edit address entry'),
                command = self.edit_execute
                )

            self.dialog.withdraw()

            self.aliasentry = Pmw.EntryField(self.dialog.interior(),
                labelpos = 'w',
                label_text = _('Alias:'),
                value = self.addressList.getcurselection()[0][0]
                )

            self.aliasentry.pack(expand = 1, fill = 'both', padx = 0, pady = 0)

            self.addressentry = Pmw.ScrolledText(self.dialog.interior(),
                labelpos = 'n',
                label_text = _('Address:'),
                )
            self.addressentry.insert(END,
                re.sub(',', '\n', self.addressList.getcurselection()[0][1]))


            self.addressentry.pack(expand = 1, fill = 'both', padx = 0, pady = 0)

            self.altfromframe = Frame(self.dialog.interior())
            self.altfromflag = IntVar()
            try:
                self.altfromflag.set(self.db[self.aliasentry.get()]['altfrom'])
            except KeyError:
                pass
            self.altfromcheck = Checkbutton(self.altfromframe,
                                            text=_('Use as alternate from'),
                                            justify=LEFT,
                                            anchor='w',
                                            command = self.altfromcheckCmd,
                                            variable=self.altfromflag,)
            self.altfromframe.pack(side='top', fill='both', padx = 0, pady = 0)
            self.altfromcheck.pack(side='left', padx = 0, pady = 0)

            self.signframe = Frame(self.dialog.interior())
            self.signtextlabel = Label(self.signframe,
                                    justify=RIGHT,
                                    anchor='w',
                                   text=_('Signature file: '))
            self.signtextlabel.pack(side = 'left', padx = 0, pady = 0)

            try:
                self.signfn.set(self.db[self.aliasentry.get()]['sign'])
            except KeyError:
                pass
            self.signfnlabel = Button(self.signframe,
                                    justify=LEFT,
                                    anchor='w',
                                    relief = RIDGE,
                                    command= self.edit_signfn,
                                    textvariable=self.signfn)

            self.signfnlabel.pack(expand = 1, side = 'right', fill = 'both', padx = 0, pady = 0)
            self.altfromcheckCmd()
            self.result = self.dialog.activate()


    def altfromcheckCmd(self):
        if self.altfromflag.get() == 1:
            self.signframe.pack(expand = 1, side='bottom', fill = 'both', padx = 0, pady = 0)
        else:
            self.signframe.pack_forget()


    def edit_execute(self, result):
        if  result == 'OK':
            addr = re.sub('\n', ',', self.addressentry.get())
            if addr[len(addr) -1] == ',':
                addr = addr[:len(addr)-2]
            addr = re.sub(';;', ',', addr)
            self.addressList.delete(self.cursel)
            self.addressList.insert(self.cursel,
                    [self.aliasentry.get(), addr])

            self.db[self.aliasentry.get()]['altfrom'] = self.altfromflag.get()
            s = self.signfn.get()
            if s != '':
                self.db[self.aliasentry.get()]['sign'] = s

        self.dialog.deactivate(result)


    def edit_signfn(self):
        fn = tkFileDialog.askopenfilename(
                            filetypes=[("all filez", "*")],
                            title=_('Select signature file'))

        self.signfn.set(fn)

 
    def get(self):
        return self.addressWidget.get()

#########################################################################

def gpgVersion():
    """
    Return gpg version number, None if gpg not found
    """

    f =  os.popen('gpg --version')
    s = f.read()
    f.close()
    po = re.compile('gpg \(GnuPG\) (.+)')
    mo = po.match(s)
    if mo != None:
        return mo.groups()[0]
    else:
        return None


#########################################################################

class ConfWidget(Pmw.MegaWidget):

    def __init__(self, parent, c, server = None):
        l1 =  c.sections()
        l2 = []
        l = []

        for s in l1:
            mo = re.match('IMAPFOLDER_(.+)', s)
            if mo != None:
                m =  mo.groups()[0]
                s = re.sub('_SLASH_', '/', m)
                s = re.sub('_TILDE_', '~', s)
                s = 'FOLDER ' + s
            l.append(s)
        l.sort()
        
        self.parent = parent
        self.conf = c
        self.server = server
        
        self.dialog = Pmw.SelectionDialog(parent,
           title = _('My SelectionDialog'),
           buttons = ('Ok', 'Folder add', 'Folder remove', 'Cancel'),
           defaultbutton = 'Ok',
           scrolledlist_labelpos = 'n',
           label_text = _('Select section:'),
           scrolledlist_items = l,
           command = self.execute,
           listbox_borderwidth=0,
           listbox_selectbackground='gray95',
           listbox_selectborderwidth=2,
           listbox_background='gray85',
           listbox_relief=RAISED)

        self.dialog.activate()


    def execute(self, result):

        sels = self.dialog.getcurselection()
        if result == 'Ok':
            if len(sels) == 0:
                print 'You clicked on', result, '(no selection)'
            else:
                #print 'You clicked on', result, sels
                s = sels[0]
                try:
                    if s.split()[0] == 'FOLDER':
                        s = s.split()[1]
                        s = s = re.sub('/', '_SLASH_', s)
                        s = re.sub('~', '_TILDE_', s)
                        s = 'IMAPFOLDER_' + s
                except IndexError:
                    pass
                
                nw = ConfSelectOptionWidget(self.parent, self.conf, s, sels[0])
        elif result == 'Folder add':
            #print 'You clicked on Add folder config'
            l = self.server.getSubscribedFolderList()
            nw = ConfAddFolderSectionWidget(self.parent, self.conf, l)
            self.dialog.deactivate(result)
            self.__init__(self.parent, self.conf, self.server)
        elif result == 'Folder remove':
            s = sels[0]
            #print 'You clicked on Remove folder', s, 'config'
            if s.split()[0] == 'FOLDER':
                s = s.split()[1]
                if self.confirmDialog('Remove config for folder ' + s + '?'):
                    s = re.sub('/', '_SLASH_', s)
                    s = re.sub('~', '_TILDE_', s)
                    s = 'IMAPFOLDER_' + s
                    try:
                        self.conf.remove_section(s)
                    except ConfigParser.NoSectionError:
                        print 'No folder section', s
            else:
                print s, 'is not a folder'
                        
            self.dialog.deactivate(result)
            self.__init__(self.parent, self.conf, self.server)                    
        else:
            self.dialog.deactivate(result)


    def confirmDialog(self,s):
        dialog = Pmw.MessageDialog(self.parent,
            title = _('Confirm'),
            defaultbutton = 0,
            buttons = (_('Ok'), _('Cancel')),
            message_text = s,
            )
        dialog.iconname('Confirm dialog')
        result = dialog.activate()
        return (result == _('Ok'))

#########################################################################

class ConfSelectOptionWidget(Pmw.MegaWidget):

    def __init__(self, parent, c, s, sn):

        self.parent = parent
        self.conf = c
        self.section = s
        self.sectionname = sn
        #print 'Section', s

        l = c.options(s)
        #print 'Options', l
        l.sort()
        
        for o in l:
            l[l.index(o)] = o + ': ' + c.get(self.section, o)

        self.dialog = Pmw.SelectionDialog(parent,
           title = _('My Option SelectionDialog'),
           buttons = ('Ok', 'Cancel'),
           defaultbutton = 'Ok',
           scrolledlist_labelpos = 'n',
           label_text = self.sectionname + '\n' + _('Select option:') ,
           scrolledlist_items = l,
           command = self.execute,
           listbox_borderwidth=0,
           listbox_selectbackground='gray95',
           listbox_selectborderwidth=2,
           listbox_background='gray85',
           listbox_relief=RAISED)

        self.dialog.activate()


    def execute(self, result):

        sels = self.dialog.getcurselection()
        if result == 'Ok':
            if len(sels) == 0:
                print 'You clicked on', result, '(no selection)'
            else:
                o =  sels[0]
                on = o.split(':')[0]
                ov = o.split(':')[1]
                #print 'You clicked on', result, ': ', self.conf.get(self.section, on)
                nw = ConfSetOptionWidget(self.parent, self.conf, self.section, self.sectionname, on, ov)
                self.dialog.deactivate(result)
                self.__init__(self.parent, self.conf, self.section, self.sectionname, self.server)
                
        else:
            self.dialog.deactivate(result)


##############################################################################
class ConfSetOptionWidget:
    def __init__(self, parent, c, s, sn, opt, val):
        self.parent = parent
        self.conf = c
        self.sect = s
        self.opt = opt
        self.val = val
        
        # Create the dialog.
        self.dialog = Pmw.Dialog(self.parent,
            buttons = ('OK', 'Cancel'),
            defaultbutton = 'OK',
            title = _('Set option value'),
            command = self.execute)

        self.dialog.withdraw()

        self.entry_ = Pmw.EntryField(self.dialog.interior(),
                        labelpos = 'w',
                        label_text = 'Section ' + sn +'\n' + opt + ':' ,
                        value = val,
                        )

        self.entry_.pack(expand = 1, fill = 'both', padx = 0, pady = 0)
        self.result = self.dialog.activate()


    def execute(self, result):
        if  result == 'OK':
            self.conf.set(self.sect, self.opt, self.entry_.get())

        self.dialog.deactivate(result)

##############################################################################
class ConfAddFolderSectionWidget:
    def __init__(self, parent, c, l = None):
        self.parent = parent
        self.conf = c

        self.dialog = Pmw.SelectionDialog(parent,
            title = _(' Add folder configration'),
            buttons = ('Ok', 'Cancel'),
            defaultbutton = 'Ok',
            scrolledlist_labelpos = 'n',
            label_text = _('Folder:'),
            scrolledlist_items = l,
            command = self.execute,
                    listbox_borderwidth=0,
                    listbox_selectbackground='gray95',
                    listbox_selectborderwidth=2,
                    listbox_background='gray85',
                    listbox_relief=RAISED
            )

        self.dialog.activate()



    def execute(self, result):
        if  result == 'Ok':
            s = self.dialog.getcurselection()[0]
            #print 'selection:', s
            s = s = re.sub('/', '_SLASH_', s)
            s = re.sub('~', '_TILDE_', s)
            s = 'IMAPFOLDER_' + s
            try:
                self.conf.add_section(s)
                self.conf.set(s, 'expire', '-')
                self.conf.set(s, 'maxsize', '-')
            except ConfigParser.DuplicateSectionError:
                print 'Folder already exists in config'
            
        self.dialog.deactivate(result)

#########################################################################

def confSave():
    fp = open(confdir + 'mamamua/mamamuarc',"w" )
    conf.write(fp)
    fp.close()
    
#########################################################################
#
#
#
#########################################################################


# Init
root = Tk()
root.title('Mama Mua')

conf = ConfigParser.ConfigParser()

try:
    confdir = os.environ['HOME']
except KeyError:
    confdir = os.getcwd()

if (confdir[len(confdir) -1] != '/') and (confdir[len(confdir) -1] != '\\'):
    confdir = confdir + "/"

try:
    os.stat(confdir + 'mamamua/mamamuarc' ) 
    conf.read(confdir + 'mamamua/mamamuarc' )
except OSError:
    print 'No config file'
    conf.add_section('MAIN')
    conf.add_section('USER')
    conf.add_section('IMAP')
    conf.add_section('SMTP')
    conf.add_section('PRINT')

if not conf.has_section('MAIN'):
    print '-----------------------------------------------------------'
    print 'Missing section MAIN in ' + confdir + 'mamamua/mamamuarc.'
    print 'Can be old config file format with DEFAULT section,'
    print 'try to rename DEFAULT to MAIN and study new config example.'
    print '-----------------------------------------------------------'    
    conf.add_section('MAIN')

    
try:
    MIMETYPES = conf.get("MAIN", "mimetypes")
    mimetypes.init([MIMETYPES])
except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
    MIMETYPES = '-'
    if hasattr(conf, 'set'):
        conf.set('MAIN', 'mimetypes', MIMETYPES)  
    
try:
    printcommand =  conf.get("PRINT", "command")
    if printcommand == '-':
        printcommand = None
except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
    printcommand = None
    print "No printer"
    if hasattr(conf, 'set'):
        conf.set('PRINT', 'command', '-')    

try:
    LOCALEDIR = conf.get("MAIN", "localedir")
except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
    LOCALEDIR = '/usr/share/locale'
    print 'Default LOCALEDIR:', LOCALEDIR
    if hasattr(conf, 'set'):
        conf.set('MAIN', 'localedir', LOCALEDIR)
try:
    LANG = conf.get("MAIN", "language")
    os.environ['LANG'] = LANG
except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
    try:
        LANG = os.environ['LANG']
    except KeyError:
        LANG = 'en_US'
        print 'Default LANG:', LANG
        os.environ['LANG'] = LANG
    if hasattr(conf, 'set'):
        conf.set('MAIN', 'language', LANG)        

import gettext # Don't import gettext until the LANG environment variable is updated
gettext.bindtextdomain(PACKAGE_, LOCALEDIR)
gettext.textdomain(PACKAGE_)
_ = gettext.gettext

#print _('Localized Mama Mua')

# Read help file
try:
    s = sys.argv[0]
    s = s[0 : string.rfind(s,'/') + 1] +LANG+'/commandhelp'
    #s = confdir+'mamamua/'+LANG+'/commandhelp'
    f = open(s)
    commands_text = re.sub('\r', '', f.read())
    f.close()
except (OSError, IOError):
    print 'No file ', s
    print 'LOCALEDIR', LOCALEDIR, ', LANG', LANG
    try:
        s = LOCALEDIR + '/' + LANG + '/commandhelp' 
        f = open(s)
        commands_text =  re.sub('\r', '', f.read())
        f.close()
    except(OSError, IOError):
        print 'No help file in ', s, ', default english command help'
        pass
    
server = ImapServer(root, config = conf, confdir = confdir)

if server.getStatus() != 'UP':
    server = NullMailServer(root, config = conf, confdir = confdir)

widget = MailWidget(root, mailserver = server, confdir = confdir)

try:
    if sys.argv[1][0] == '-':
        s = sys.argv[1][2:]
        t = sys.argv[2]
    else:
        s = ''
        t = sys.argv[1]

    ComposeMail(root, mailserver=server, confdir = confdir, toAddress=t, toSubject=s)
except IndexError:
    pass

root.mainloop()

widget.saveGeometry()
print 'Exit Mama Mua'

